# -*- coding: utf-8 -*-

import os
import pwd
import time
import logging
import smtplib
import platform
import mimetypes
import shutil
import uuid
from email import encoders
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from d3r.celpp.filetransfer import FtpFileTransfer
from d3r.celpp import util


logger = logging.getLogger(__name__)


class D3RParameters(object):
    """Holds parameters common to Tasks

    """
    pass


class TaskException(Exception):
    """base exception class for all the other exceptions provided by
       this module.
    """
    pass


class UnsetPathError(TaskException):
    """Exception to denote path is unset
    """
    pass


class UnsetFileNameError(TaskException):
    """Exception to denote file name is unset
    """
    pass


class UnsetNameError(TaskException):
    """Exception to denote name is unset
    """
    pass


class UnsetCommandError(TaskException):
    """Exception to denote command is unset
    """
    pass


class UnsetStageError(TaskException):
    """Exception to denote stage is unset
    """
    pass


class TaskUnableToStartError(TaskException):
    """Exception to denote when task cannot start due to failure
    """
    pass


class TaskFailedError(TaskException):
    """Exception to denote when a task failed
    """
    pass


class EmailSendError(TaskException):
    """Exception to denote when SmtpEmailer was unable to send an email
    """
    pass


class D3RTask(object):
    """Represents a base Task that can be run.

    This is a base class from which other classes that actual do work
    can be derived from.  It provides internal attributes for name,
    stage, and status.  As well as a run() function.

    """

    # Put at start of subject line in all emails sent from D3RTask
    SUBJECT_LINE_PREFIX = "[d3rcelpp] "

    STAGE_DIRNAME_PREFIX = "stage"

    START_FILE = "start"
    ERROR_FILE = "error"
    COMPLETE_FILE = "complete"

    START_STATUS = "start"
    COMPLETE_STATUS = "complete"
    UNKNOWN_STATUS = "unknown"
    NOTFOUND_STATUS = "notfound"
    ERROR_STATUS = "error"
    STDERR_SUFFIX = ".stderr"
    STDOUT_SUFFIX = ".stdout"
    TMP_DIR_SUFFIX = '.tmpdir'
    MAX_CHARS_FOR_EMAIL_STR = 250000
    TEXT_TRUNCATED_STR = 'TEXT TRUNCATED\n'

    def __init__(self, path, args):
        """Constructor

        Creates a `D3RTask` with `D3RTask.UNKNOWN_STATUS` status
        with `path` and `args` set to values passed in
        """
        self._path = path
        self._name = None
        self._stage = None
        self._status = D3RTask.UNKNOWN_STATUS
        self._error = None
        self._args = args
        self._can_run = None
        self._start_time = None
        self._duration = -1
        self._email_log = None

        try:
            logger.debug('ftpconfig set to ' + args.ftpconfig)
            self._file_uploader = FtpFileTransfer(args.ftpconfig)
        except:
            logger.debug('FtpFileUploader not set.  This may not be '
                         'an error')
            self._file_uploader = None

    def get_args(self):
        return self._args

    def set_args(self, args):
        self._args = args

    def set_email_log(self, email_log):
        self._email_log = email_log

    def append_to_email_log(self, append_log):
        if self._email_log is None:
            self._email_log = append_log
        else:
            self._email_log += append_log

    def get_email_log(self):
        return self._email_log

    def get_path(self):
        return self._path

    def set_path(self, path):
        self._path = path

    def get_name(self):
        return self._name

    def set_name(self, name):
        self._name = name

    def get_stage(self):
        return self._stage

    def set_stage(self, stage):
        self._stage = stage

    def get_status(self):
        return self._status

    def set_status(self, status):
        self._status = status

    def get_error(self):
        return self._error

    def set_error(self, error):
        self._error = error

    def get_file_transfer(self):
        """Gets the file uploader
        """
        return self._file_uploader

    def set_file_transfer(self, file_uploader):
        """Sets file uploader
        """
        self._file_uploader = file_uploader

    def get_uploadable_files(self):
        """Returns a list of files that can be uploaded to remote server

           Provides a way for a D3RTask to define what files should
           uploaded to remote server by any D3RTaskUploader.  The default
           implementation returns an empty list
           :returns: list of files that can be uploaded.  The files should
           have full paths
        """
        file_list = []

        try:
            out_dir = self.get_dir()
        except:
            logger.exception('Unable to get dir for task')
            return file_list
        try:
            for entry in os.listdir(out_dir):
                if not os.path.isfile(os.path.join(out_dir, entry)):
                    continue

                if entry.endswith(D3RTask.STDERR_SUFFIX):
                    file_list.append(os.path.join(out_dir, entry))
                elif entry.endswith(D3RTask.STDOUT_SUFFIX):
                    file_list.append(os.path.join(out_dir, entry))

            error_file = os.path.join(out_dir, D3RTask.ERROR_FILE)
            if os.path.isfile(error_file):
                logger.debug('Found error file adding to list')
                file_list.append(error_file)

        except OSError:
            logger.warning('Caught exception looking for error and '
                           'stderr/stdout files')

        return file_list

    def _get_smtp_server(self):
        """Gets smtplib server for localhost

           creates smtplib.SMTP server using get_args().smtp and
           get_args().smtpport for the smtp server and port
        :return: SMTP server
        """
        return smtplib.SMTP(self.get_args().smtp,
                            self.get_args().smtpport)

    def _build_from_address(self):
        """Returns from email address

           :return: string in format login@host
        """
        hostname = platform.node()
        if hostname is '':
            hostname = 'localhost'
        return (pwd.getpwuid(os.getuid())[0] + '@' + hostname)

    def _get_program_name(self):
        """Gets name of program running

        Looks at _args.program for the program name, if unset or
        empty then __file__ is used
        :return: Name of program that invoked this
        """
        program_name = __file__
        try:
            program_name = self.get_args().program
        except:
            logger.debug('args.program  was not set.  '
                         'using __file__')
        return program_name + ' ' + self._get_program_version()

    def _get_program_version(self):
        """Gets version of program running
        This is done by looking at _args.version
        :return: Version of program or empty string if unset
        """
        try:
            version = self.get_args().version
        except AttributeError:
            version = ''
            logger.debug('args.version was not set')
        return str(version)

    def _get_time(self):
        """ Gets the current time as a string using ctime()
        :return: ctime string followed by --
        """
        return time.ctime() + ' -- '

    def _send_start_email(self):
        """Creates start email message that can be passed to sendmail

        """
        email_log = ''
        if self._email_log is not None:
            email_log = (self._email_log + '\n\n')

        the_message = ('Hi,\n ' + self.get_dir_name() + ' has started' +
                       ' running on host ' + platform.node() +
                       '\nunder path ' + self.get_dir() + '\n\n' + email_log +
                       'Sincerely,\n\n' + self._get_program_name())

        msg = MIMEText(the_message)
        msg['Subject'] = (D3RTask.SUBJECT_LINE_PREFIX + self._get_time() +
                          self.get_dir_name() +
                          ' has started running')
        try:
            self._send_email(msg, self.get_args().email)
        except AttributeError as e:
            logger.debug(str(e) + ' email not set. skipping')

    def _send_end_email(self):
        """Creates end email message that can be passed to sendmail

        """

        email_log = ''
        if self._email_log is not None:
            email_log = (self._email_log + '\n\n')

        error_msg = ''
        if self.get_error() is not None:
            error_msg = ('Error:\n' + self.get_error() + '\n\n')

        the_message = ('Hi,\n ' + self.get_dir_name() + ' has finished' +
                       ' after ' + str(self._duration) + ' seconds' +
                       ' with status ' + self.get_status() + ' on host '
                       + platform.node() +
                       '\nunder path ' + self.get_dir() + '\n\n' + email_log +
                       error_msg + 'Sincerely,\n\n' + self._get_program_name())

        msg = MIMEText(the_message)
        msg['Subject'] = (D3RTask.SUBJECT_LINE_PREFIX + self._get_time() +
                          self.get_dir_name() +
                          ' has finished with status ' + self.get_status())
        try:
            try:
                self._send_email(msg, self.get_args().email)
            except AttributeError as e:
                logger.debug(str(e) + ' email not set. skipping')

            if self.get_error() is not None:
                logger.debug("Sending summary email cause task failed")
                try:
                    self._send_email(msg, self.get_args().summaryemail)
                except AttributeError as e:
                    logger.debug(str(e) + ' email not set. skipping')

        except Exception:
            logger.exception('Caught exception trying to send end email, '
                             'skipping email notification')

    def _send_email(self, mime_msg, email_csv):
        """Sends email with message passed in


           If email attribute is set with valid email addresses
           in get_args() object then this method will send
           an email with `message` passed in as content.
        """
        try:
            if email_csv is not None:

                split_email_list = email_csv.split(',')

                logger.debug('Sending email to: ' +
                             ", ".join(split_email_list))
                server = self._get_smtp_server()

                from_addr = self._build_from_address()

                mime_msg['From'] = from_addr
                mime_msg['To'] = ", ".join(split_email_list)

                if self.get_args().loglevel == 'DEBUG':
                    server.set_debuglevel(1)

                server.sendmail(from_addr,
                                email_csv, mime_msg.as_string())
                server.quit()
                return
            else:
                logger.debug('Email was set to None, skipping email ' +
                             'notification')
        except AttributeError as e:
            logger.debug('No email passed in, skipping email notification ' +
                         str(e))

    def _upload_task(self):
        """Uploads files related to task with FileUploader passed in


           :return: None
        """
        if self.get_file_transfer() is None:
            return
        try:
            uploadable_files = self.get_uploadable_files()
            self.get_file_transfer().connect()
            self.get_file_transfer().upload_files(uploadable_files)
            summary = self.get_file_transfer().get_upload_summary()
            self.append_to_email_log('\n' + summary + '\n')
        except:
            logger.exception('Caught exception trying to upload files')
        finally:
            self.get_file_transfer().disconnect()

    def start(self):
        """Denotes start of task

           Creates task directory, along with token start file.
           If get_args() has email attribute then an email is
           sent denoting task start.  Also sets get_status()
           to D3RTask.START_STATUS.
           If there is an error creating task directory then
           end() is invoked and get_error() will be set with
           message denoting error
        """
        self._start_time = int(time.time())
        logger.info(self.get_dir_name() + ' task has started ')

        self.set_status(D3RTask.START_STATUS)

        self._send_start_email()

        try:
            self.create_dir()
        except Exception as e:
            self.set_error('Caught exception trying to create directory ' +
                           str(e))
            self.end()
            return

        sfile = open(os.path.join(self.get_dir(),
                                  D3RTask.START_FILE), 'w')
        sfile.write(self._get_program_version())
        sfile.flush()
        sfile.close()

    def end(self):
        """Denotes task has completed

           If get_error() is not none then 'error' token file is
           put in task directory and get_status() set to D3RTask.ERROR_STATUS
           Otherwise 'complete' token file is put in task directory.
           if get_args() email is set then a notification of task
           completion is sent
        """

        if self._start_time is not None:
            self._duration = int(time.time()) - self._start_time
        else:
            self._duration = -1

        logger.info(self.get_dir_name() + ' task took ' +
                    str(self._duration) + ' seconds and has ' +
                    'finished with status ' + self.get_status())

        if self.get_error() is not None:
            logger.error(self.get_dir() + ' task failed with error ' +
                         self.get_error())
            self.set_status(D3RTask.ERROR_STATUS)
            try:
                f = open(os.path.join(self.get_dir(),
                                      D3RTask.ERROR_FILE), 'a')
                f.write(self.get_error() + '\n')
                f.flush()
                f.close()
            except:
                logger.exception('Caught exception')

        self._upload_task()

        if self.get_status() == D3RTask.ERROR_STATUS:
            try:
                err_file = os.path.join(self.get_dir(),
                                        D3RTask.ERROR_FILE)
                if not os.path.isfile(err_file):
                    open(err_file, 'a').close()
            except:
                logger.exception('Caught Exception')
            self._send_end_email()
            return

        self.set_status(D3RTask.COMPLETE_STATUS)
        open(os.path.join(self.get_dir(),
                          D3RTask.COMPLETE_FILE), 'a').close()
        self._send_end_email()

    def can_run(self):
        """Always returns False

           Derived classes should override this method
        """
        self._can_run = False
        return False

    def run(self):
        """Sees if task can run

           This is the base implementation and does not actually
           run anything.  Instead this method calls can_run()
           if the internal variable _can_run is None.  It is assumed
           can_run() will set _can_run variable.  If _can_run is False
           method will just return.  If _can_run is True then start()
           is called and after invocation get_error() is checked, if
           an error is found end() is called and _can_run is set to False
        """
        name = self._name
        if name is None:
            name = 'unset'

        logger.info(name + ' task is running')
        if self._can_run is None:
            logger.info("Running can_run() to check if its allright " +
                        "to run")
            self.can_run()

        if self._can_run is False:
            logger.info("can_run() came back with false cannot run")
            if self.get_error() is not None:
                self.end()
            return

        self.start()

        if self.get_error() is not None:
            self._can_run = False
            self.end()
            return

    def get_dir_name(self):
        """Gets directory name for task

           :raises: UnsetStageError, UnsetNameError
        """
        if self._stage is None:
            raise UnsetStageError('Stage must be set')

        if self._name is None:
            raise UnsetNameError('Name must be set')

        return (D3RTask.STAGE_DIRNAME_PREFIX + "." + str(self._stage) +
                "." + self._name)

    def get_dir(self):
        """Gets full path of Task

           :raises: UnsetPathError if path is not set,
                    and all exceptions from get_dir_name()
        """
        if self.get_path() is None:
            raise UnsetPathError('Path must be set')

        return os.path.join(self.get_path(),
                            self.get_dir_name())

    def update_status_from_filesystem(self):
        """Updates status by querying filesystem.

           Sets status based on contents of path on filesystem.
           If path does not exist then status is NOTFOUND_STATUS.
           If complete file exists under path then status is COMPLETE_STATUS
           If error file exists under path then status is ERROR_STATUS
           If start file exists under path then status is START_STATUS
           else status is UNKNOWN_STATUS
           :return: status as string updated from file system
           """
        if self.get_path() is None:
            raise UnsetPathError('Path must be set')

        path_to_check = self.get_dir()
        try:
            logger.debug('Checking path ' + path_to_check)
        except:
            pass

        self.set_status(self._get_status_of_task_in_dir(path_to_check))
        logger.debug(self.get_name() + ' task status set to ' +
                     self.get_status())
        return self.get_status()

    def create_dir(self):
        """Creates directory for task.

           Directory will be named stage.<stage>.<name>
           and located under get_path()
           :raises: UnsetPathError if path is not set and all exceptions
                    from get_dir_name(), and OSError if there is a
                    problem creating the directory
           :return: path to created directory
           """
        the_path = self.get_dir()

        logger.debug('Creating directory: ' + the_path)

        os.mkdir(the_path)

        return the_path

    def write_to_file(self, str, file_name):
        """Writes `str` to `file_name` under `task` directory

           If `str` is None file is created but nothing is written.
           File is written using 'w' mode so if file exists it will
           be overwritten.

           :raises: UnsetFileNameError if file_name is None
        """
        if file_name is None:
            raise UnsetFileNameError('file_name must be set')
        file_to_write = os.path.join(self.get_dir(), file_name)
        logger.debug('Writing file ' + file_to_write)
        f = open(file_to_write, 'w')

        if str is not None:
            f.write(str)

        f.close()

    def _get_status_of_task_in_dir(self, path):
        """Gets status of task based on existance of files in path

           Examines get_path() and returns status based on the following
           conditions
           :return: status of task as string
        """
        if not os.path.isdir(path):
            return D3RTask.NOTFOUND_STATUS

        if os.path.isfile(os.path.join(path, D3RTask.COMPLETE_FILE)):
            return D3RTask.COMPLETE_STATUS

        if os.path.isfile(os.path.join(path, D3RTask.ERROR_FILE)):
            return D3RTask.ERROR_STATUS

        if os.path.isfile(os.path.join(path, D3RTask.START_FILE)):
            return D3RTask.START_STATUS

        return D3RTask.UNKNOWN_STATUS

    def run_external_command(self, command_name, cmd_to_run,
                             command_failure_is_fatal,
                             timeout=None,
                             kill_delay=10,
                             polling_sleep_time=1):
        """Runs external command line process

        Method runs external process logging the command
        run to email log.  Standard output and error are
        written to current working directory `command_name`.stdout
        and `command_name`.stderr respectively and if process.

        :param command_name: Name of command, human readable version
                             used as prefix for .stderr and .stdout
                             files
        :param cmd_to_run: command with arguments to run,
                        passed to subprocess.Popen
        :param command_failure_is_fatal: True means if process
               fails with nonzero exit code or there is an
               exception then status of task is set to
               D3RTask.ERROR_STATUS and failure message appended
               to email log and task.set_error is set
        :return: exit code of process
        :raise: UnsetNameError if command_name is None
        :raise: UnsetCommandError if cmd_to_run is None
        """
        if command_name is None:
            raise UnsetNameError('Command name must be set')

        if cmd_to_run is None:
            raise UnsetCommandError('Command must be set')

        if command_failure_is_fatal is None:
            command_failure_is_fatal = True

        logger.info("Running command " + cmd_to_run)

        self.append_to_email_log('Running command: ' + cmd_to_run + '\n')
        cmd_tmp_dir = None
        try:
            if timeout is not None:
                cmd_tmp_dir = os.path.join(self.get_dir(),
                                           command_name + str(uuid.uuid1()) +
                                           D3RTask.TMP_DIR_SUFFIX)
                os.makedirs(cmd_tmp_dir, mode=0o0775)
                pst = polling_sleep_time  # trying to make flake8 happy

                returncode, out, err = util.\
                    run_external_command_with_timeout(cmd_to_run,
                                                      cmd_tmp_dir,
                                                      timeout=timeout,
                                                      kill_delay=kill_delay,
                                                      polling_sleep_time=pst)
            else:
                returncode, out, err = util.run_external_command(cmd_to_run)
        except Exception as e:
            logger.exception("Error caught exception")
            self.set_status(D3RTask.ERROR_STATUS)
            self.set_error("Caught Exception trying to run " +
                           cmd_to_run + " : " + str(e))
            self.end()
            return 1
        finally:
            if cmd_tmp_dir is not None:
                shutil.rmtree(cmd_tmp_dir)

        self.write_to_file(err, command_name + D3RTask.STDERR_SUFFIX)
        self.write_to_file(out, command_name + D3RTask.STDOUT_SUFFIX)

        if returncode != 0:
            if command_failure_is_fatal:
                self.set_status(D3RTask.ERROR_STATUS)
                self.set_error("Non zero exit code: " + str(returncode) +
                               " received. Standard out: " + out +
                               " Standard error: " + err)
            else:
                max_chars = D3RTask.MAX_CHARS_FOR_EMAIL_STR
                trunc_out = self._get_email_truncated_string(out,
                                                             max_chars)
                trunc_err = self._get_email_truncated_string(err,
                                                             max_chars)

                self.append_to_email_log("Although considered non fatal " +
                                         "for processing of stage a " +
                                         "non zero exit code: " +
                                         str(returncode) +
                                         "received. Standard out: " +
                                         trunc_out +
                                         " Standard error : " + trunc_err)
        return returncode

    def _get_email_truncated_string(self, val, max_chars):
        """Truncates string so it fits within a certain number of characters
        This method will remove characters from the start of the string so
        it fits within the `max_chars` limit and prefixed with value of
        `D3RTask.TEXT_TRUNCATED_STR` to the start.
        NOTE: the prefix string is not counted toward the max chars
        :param val: String to truncate
        :params max_chars: Maximum length of string
        :return: Truncated `val` string with `D3RTask.TEXT_TRUNCATED_STR`
                 prefixed
        """
        if val is None:
            logger.warning('String passed in was None so returning None')
            return None

        if max_chars is None or max_chars < 0:
            logger.debug('max_chars is set to None or is negative, '
                         'just returning string')
            return val

        str_len = len(val)
        logger.debug('String length is: ' + str(str_len))
        if str_len <= max_chars:
            return val

        trunc_start = str_len - max_chars
        logger.debug('Truncating string at index: ' + str(trunc_start))
        return (D3RTask.TEXT_TRUNCATED_STR +
                val[trunc_start:])


class Attachment(object):
    """Class to hold file to attach to email
    """
    def __init__(self, file_to_attach, desired_name):
        self._file_to_attach = file_to_attach
        self._desired_name = desired_name

    def get_file_to_attach(self):
        return self._file_to_attach

    def get_desired_name(self):
        return self._desired_name


class SmtpEmailer(object):
    """Simple Wrapper class to send email via smtplib
    """

    def __init__(self, smtp_host='localhost', port=25):
        """Constructor
        :param smtp_host: host of stmp server
        :param port: port stmp server is on
        """
        self._smtp_host = smtp_host
        self._port = port
        self._alt_smtp_server = None

    def set_alternate_smtp_server(self, server):
        """Sets alternate smtp server to use
        """
        self._alt_smtp_server = server

    def send_email(self, from_address, to_list, subject, message,
                   reply_to=None, attachments=None):
        """Sends email
        :param from_address: from email address
        :param to_list: list of email addresses as strings to send email to
        :param subject: Subject of email
        :param message: Body of email
        :param reply_to: optional parameter to alter the reply to email
        :param attachments: optional list parameter containing Attachment
        objects
        representing files to attach to email
        :param htmlmessage: optional parameter containing html version of
        email body
        :raises EmailSendError: if there was an error creating or sending
        the email
        """
        server = None
        try:
            mime_msg = self._build_mime_message(from_address, to_list,
                                                subject, message, reply_to,
                                                attachments)
            server = self._get_server()
            server.sendmail(from_address, to_list, mime_msg.as_string())
        except smtplib.SMTPConnectError as e:
            logger.exception('Caught exception')
            raise EmailSendError('Unable to connect to smtp server ' + str(e))
        except Exception as e:
            logger.exception('Caught exception')
            raise EmailSendError('Caught exception ' + str(e))
        finally:
            if server is not None:
                server.quit()

    def generate_from_address_using_login_and_host(self):
        """Creates from email address from login and hostname
        of machine running this script.
        :returns: from email address as string
        """
        hostname = platform.node()
        if hostname is '':
            hostname = 'localhost'
        return pwd.getpwuid(os.getuid())[0] + '@' + hostname

    def _get_server(self):
        """Gets Smtp server
        :returns: smtplib.SMTP object unless set_alternate_smtp_server
        was set in
        which case that object is returned
        """
        if self._alt_smtp_server is not None:
            return self._alt_smtp_server

        return smtplib.SMTP(self._smtp_host,
                            self._port)

    def _append_attachments(self, msg_root, attachments):
        """Attaches attachments to `msg_root`
        :param msg_root: MIMEMultipart message to attach the files to
        :param attachments: List of Attachment objects to attach to msg_root
        :returns msg_root: with attachments added
        """
        if attachments is None:
            return msg_root

        for a in attachments:
            fname = a.get_file_to_attach()
            if not os.path.isfile(fname):
                logger.error('File does not exist: ' + fname)
                continue
            ctype, encoding = mimetypes.guess_type(fname)
            if ctype is None or encoding is not None:
                # go with binary type if we did not get a guess
                ctype = 'application/octet-stream'
            maintype, subtype = ctype.split('/', 1)
            if maintype == 'text':
                logger.debug('Attaching text file ' + fname)
                fp = open(fname)
                attach = MIMEText(fp.read(), _subtype=subtype)
                fp.close()
            elif maintype == 'image':
                logger.debug('Attaching image file ' + fname)
                fp = open(fname, 'rb')
                attach = MIMEImage(fp.read(), _subtype=subtype)
                fp.close()
            else:
                logger.debug('Attaching unknown file ' + fname)
                fp = open(fname, 'rb')
                attach = MIMEBase(maintype, subtype)
                attach.set_payload(fp.read())
                fp.close()
                # Encode the payload using Base64
                encoders.encode_base64(attach)
            # Set the filename parameter
            if a.get_desired_name() is None:
                filename = os.path.basename(fname)
            else:
                filename = a.get_desired_name()
            attach.add_header('Content-Disposition', 'attachment',
                              filename=filename)
            msg_root.attach(attach)

        return msg_root

    def _build_mime_message(self, from_address, to_list,
                            subject, message, reply_to, attachments):
        """Creates MIMEText object and returns it
        :param from_address: from email address
        :param to_list: list of email addresses as strings to send email to
        :param subject: Subject of email
        :param message: Body of email
        :returns MIMEText object upon success or None upon error
        """
        msg_root = MIMEMultipart("alternative")

        msg_root['Subject'] = subject
        msg_root['From'] = from_address
        msg_root['To'] = ", ".join(to_list)
        if reply_to is not None:
            msg_root.add_header('reply-to', reply_to)

        msg_root.attach(MIMEText(message, "plain"))
        updated_msg_root = self._append_attachments(msg_root, attachments)
        updated_msg_root.attach(MIMEText('<pre>\n' + message + '\n</pre>\n',
                                         "html"))
        updated_msg_root.preamble = ('You need a MIME enabled mail reader '
                                     'to see this message')

        return updated_msg_root



#! /usr/bin/env python

import sys
import os
import argparse
import psutil
import logging
from datetime import date

import d3r
from d3r.celpp import util
from d3r.celpp.task import D3RParameters


# create logger
logger = logging.getLogger('d3r.celpprunner')
DEFAULT_LOG_LEVEL = 'ERROR'
p = D3RParameters()
p.loglevel = DEFAULT_LOG_LEVEL
util.setup_logging(p)


from d3r.celpp.blastnfilter import BlastNFilterTask
from d3r.celpp.proteinligprep import ProteinLigPrepTask
from d3r.celpp.dataimport import DataImportTask
from d3r.celpp.glide import GlideTask
from d3r.celpp.evaluation import EvaluationTaskFactory
from d3r.celpp.makeblastdb import MakeBlastDBTask
from d3r.celpp.vina import AutoDockVinaTask
from d3r.celpp.postevaluation import PostEvaluationEmailer
from d3r.celpp.postevaluation import PostEvaluationTask
from d3r.celpp.challengedata import ChallengeDataTask
from d3r.celpp.chimeraprep import ChimeraProteinLigPrepTask
from d3r.celpp.filetransfer import FtpFileTransfer
from d3r.celpp.extsubmission import ExternalDataSubmissionFactory

from lockfile.pidlockfile import PIDLockFile


CREATE_CHALLENGE = 'createchallenge'
CHIMERA_PREP = 'chimeraprep'
POST_EVAL = 'postevaluation'
STAGE_FLAG = '--stage'


def _get_lock(theargs, stage):
    """Create lock file to prevent this process from running on same data.

       This uses ``PIDLockFile`` to create a pid lock file in latest_weekly
       directory named celprunner.<stage>.lockpid
       If pid exists it is assumed the lock is held otherwise lock
       is broken and recreated

       :param theargs: return value from argparse and should contain
                       theargs.latest_weekly should be set to path
       :param stage: set to stage that is being run
       :return: ``PIDLockFile`` upon success
       :raises: LockException: If there was a problem locking
       :raises: Exception: If valid pid lock file already exists
       """
    mylockfile = os.path.join(theargs.latest_weekly, "celpprunner." +
                              stage + ".lockpid")
    logger.debug("Looking for lock file: " + mylockfile)
    lock = PIDLockFile(mylockfile, timeout=10)

    if lock.i_am_locking():
        logger.debug("My process id" + str(lock.read_pid()) +
                     " had the lock so I am breaking")
        lock.break_lock()
        lock.acquire(timeout=10)
        return lock

    if lock.is_locked():
        logger.debug("Lock file exists checking pid")
        if psutil.pid_exists(lock.read_pid()):
            raise Exception("celpprunner with pid " +
                            str(lock.read_pid()) +
                            " is running")

    lock.break_lock()
    logger.info("Acquiring lock")
    lock.acquire(timeout=10)
    return lock


def set_andor_create_latest_weekly_parameter(theargs):
    """Looks at theargs parameters to get celpp week directory

       What this method does varies by values in theargs
       parameter.

       If theargs.createweekdir is set then code will determine
       current celp week ie (dataset.week.#) and create it under
       the theargs.celppdir.

       Otherwise code will find the latest
       celpp week dir under the theargs.celppdir directory and set
       theargs.latest_weekly to this path.


    """
    try:
        if theargs.createweekdir:
            celp_week = util.get_celpp_week_of_year_from_date(date.today())
            logger.debug('Request to create new directory ' +
                         os.path.join(theargs.celppdir, str(celp_week[1]),
                                      'dataset.week.'+str(celp_week[0])))
            util.create_celpp_week_dir(celp_week, theargs.celppdir)
    except AttributeError:
        pass

    try:
        if theargs.customweekdir:
            theargs.latest_weekly = theargs.celppdir
        else:
            theargs.latest_weekly = \
                util.find_latest_weekly_dataset(theargs.celppdir)
    except AttributeError:
        theargs.latest_weekly = \
            util.find_latest_weekly_dataset(theargs.celppdir)
    return theargs


def run_stages(theargs):
    """Runs all the stages set in theargs.stage parameter


       Examines theargs.stage and splits it by comma to get
       list of stages to run.  For each stage found a lock file
       is created and run_stage is invoked with theargs.latest_weekly set to
       the output of util.find_latest_weekly_dataset.  After run_stage the
       lockfile is released
       :param theargs: should contain theargs.celppdir & other params
                       set via commandline
    """
    updatedtheargs = set_andor_create_latest_weekly_parameter(theargs)

    if updatedtheargs.latest_weekly is None:
        logger.info("No weekly dataset found in path " +
                    updatedtheargs.celppdir)
        return 0
    for stage_name in updatedtheargs.stage.split(','):
        logger.info("Starting " + stage_name + " stage")
        try:
            lock = _get_lock(updatedtheargs, stage_name)

            task_list = get_task_list_for_stage(updatedtheargs, stage_name)

            # run the stage
            exit_code = run_tasks(task_list)
            if exit_code is not 0:
                logger.error('Non zero exit code from task ' + stage_name +
                             'exiting')
                return exit_code
        finally:
            # release lock
            logger.debug('Releasing lock')
            lock.release()

    return 0


def run_tasks(task_list):
    """Runs a specific stage

       Runs the tasks in task_list
       :param task_list: list of tasks to run
    """
    if task_list is None:
        logger.error('Task list is None')
        return 3

    if len(task_list) == 0:
        logger.error('Task list is empty')
        return 2

    returnval = 0

    for task in task_list:
        logger.info("Running task " + task.get_name())
        try:
            task.run()
        except Exception as e:
            logger.exception("Error caught exception")
            if task.get_error() is None:
                task.set_error('Caught Exception running task: ' + e.message)

        logger.debug("Task " + task.get_name() + " has finished running " +
                     " with status " + task.get_status())
        if task.get_error() is not None:
            logger.error('Error running task ' + task.get_name() +
                         ' ' + task.get_error())
            returnval = 1

    return returnval


def _get_set_of_email_address_from_email_flags(theargs):
    """Given theargs which contains values parsed by `argparse`
       on the command line create a unique list of email
       addresses from summaryemail and email flags. If
       neither are set return None
    :param theargs: object returned by `ArgParse`
    :returns: list of unique email addresses or None
    """
    address_list = None
    try:
        if theargs.email is not None:
            address_list = []
            address_list.extend(set(theargs.email.split(',')))
    except AttributeError:
        logger.debug('Got attribute error examining email attribute'
                     'no biggy')

    try:
        if theargs.summaryemail is not None:
            if address_list is None:
                address_list = []
            address_list.extend(set(theargs.summaryemail.split(',')))
    except AttributeError:
        logger.debug('Got attribute error examining summaryemail attribute'
                     'no biggy')

    if address_list is not None:
        return list(set(address_list))


def get_task_list_for_stage(theargs, stage_name):
    """Factory method that generates a list of tasks for given stage

       Using stage_name get the list of tasks that need to
       be run.
       :param theargs: parameters set via commandline along with
                       ``theargs.latest_weekly`` which should be set to
                       to base directory where stages will be run
       :param stage_name:  Name of stage to run
    """
    if stage_name is None:
        raise NotImplementedError('stage_name is None')

    task_list = []

    logger.debug('Getting task list for ' + stage_name)

    if stage_name == CREATE_CHALLENGE:
        task_list.append(MakeBlastDBTask(theargs.latest_weekly, theargs))
        task_list.append(DataImportTask(theargs.latest_weekly, theargs))
        task_list.append(BlastNFilterTask(theargs.latest_weekly, theargs))
        task_list.append(ChallengeDataTask(theargs.latest_weekly, theargs))

    if stage_name == 'makedb':
        task_list.append(MakeBlastDBTask(theargs.latest_weekly, theargs))

    if stage_name == 'import':
        task_list.append(DataImportTask(theargs.latest_weekly, theargs))

    if stage_name == 'blast':
        task_list.append(BlastNFilterTask(theargs.latest_weekly, theargs))

    if stage_name == 'challengedata':
        task_list.append(ChallengeDataTask(theargs.latest_weekly, theargs))

    if stage_name == 'proteinligprep':
        task_list.append(ProteinLigPrepTask(theargs.latest_weekly, theargs))

    if stage_name == 'glide':
        task_list.append(GlideTask(theargs.latest_weekly, theargs))

    if stage_name == 'vina':
        task_list.append(AutoDockVinaTask(theargs.latest_weekly, theargs))

    if stage_name == CHIMERA_PREP:
        task_list.append(ChimeraProteinLigPrepTask(theargs.latest_weekly,
                                                   theargs))
    if stage_name == 'extsubmission':
        extfac = ExternalDataSubmissionFactory(theargs.latest_weekly, theargs)
        task_list.extend(extfac.get_external_data_submissions())

    if stage_name == 'evaluation':
        # use util function call to get all evaluation tasks
        # append them to the task_list
        eval_task_factory = EvaluationTaskFactory(theargs.latest_weekly,
                                                  theargs)
        task_list.extend(eval_task_factory.get_evaluation_tasks())

    if stage_name == POST_EVAL:
        # create PostEvaluationEmailer object
        ptask = PostEvaluationTask(theargs.latest_weekly, theargs)
        a_list = _get_set_of_email_address_from_email_flags(theargs)
        pmailer = PostEvaluationEmailer(a_list,
                                        None)
        ptask.set_evaluation_emailer(pmailer)
        task_list.append(ptask)

    if len(task_list) is 0:
        raise NotImplementedError(
            'uh oh no tasks for ' + stage_name + ' stage')

    return task_list


def _parse_arguments(desc, args):
    """Parses command line arguments using argparse.
    """
    parsed_arguments = D3RParameters()

    help_formatter = argparse.RawDescriptionHelpFormatter
    parser = argparse.ArgumentParser(description=desc,
                                     formatter_class=help_formatter)
    parser.add_argument("celppdir", help='Base celpp directory')
    parser.add_argument("--email", dest="email",
                        help='Comma delimited list of email addresses '
                             'to receive an email when each task starts'
                             'and ends')
    parser.add_argument("--summaryemail", dest="summaryemail",
                        help='Comma delimited list of email addresses to'
                             'receive an email summarizing docking '
                             'evaluations and when a task fails')
    parser.add_argument("--createweekdir",
                        help='Create new celpp week directory before ' +
                             'running stages',
                        action="store_true")
    parser.add_argument("--customweekdir",
                        action='store_true',
                        help="Use directory set in celppdir instead of " +
                             "looking for latest weekdir.  NOTE: " +
                             "--createweekdir " +
                             "will create a dataset.week.# dir under celppdir")
    parser.add_argument(STAGE_FLAG, required=True,
                        help='Comma delimited list' +
                        ' of stages to run.  Valid STAGES = ' +
                        '{makedb, import, blast, challengedata,'
                        + CHIMERA_PREP +
                        ', proteinligprep, extsubmission, glide, '
                        'vina, evaluation, ' + CREATE_CHALLENGE + ', ' +
                        POST_EVAL + ',} ')
    parser.add_argument("--blastnfilter", default='blastnfilter.py',
                        help='Path to BlastnFilter script '
                             '(default blastnfilter.py)')
    parser.add_argument("--blastnfiltertimeout", default=86400, type=int,
                        help='Time in seconds script is allowed to run before'
                             'being killed (default 86400)')
    parser.add_argument("--evaluationtimeout", default=43200, type=int,
                        help='Time in seconds script is allowed to run before'
                             'SIGTERM is sent (default 43200)')
    parser.add_argument("--evaluationtimeoutkilldelay", default=120, type=int,
                        help='Time in seconds script after SIGTERM is sent'
                             'when SIGKILL is sent(default 120)')
    parser.add_argument("--postanalysis", default='postanalysis.py',
                        help='Path to PostAnalysis script '
                             '(default postanalysis.py)')
    parser.add_argument("--proteinligprep", default='proteinligprep.py',
                        help='Path to proteinligprep script (default '
                             'proteinligprep.py)')
    parser.add_argument("--genchallenge", default='genchallengedata.py',
                        help='Path to genchallengedata script '
                             '(default genchallengedata.py)')
    parser.add_argument("--chimeraprep", default='chimera_proteinligprep.py',
                        help='Path to chimera_proteinligprep script '
                             '(default chimera_proteinligprep.py)')
    parser.add_argument("--glide", default='glidedocking.py',
                        help='Path to glide docking script '
                             '(default glidedocking.py)')
    parser.add_argument("--vina", default='vinadocking.py',
                        help='Path to auto dock vina docking script '
                             '(default vinadocking.py)')
    parser.add_argument("--evaluation", default='evaluate.py',
                        help='Path to evaluation script (default evaluate.py)')
    parser.add_argument("--postevaluation", default='post_evaluation.py',
                        help='Path to post evaluation script '
                             '(default post_evaluation.py)')
    parser.add_argument("--pdbdb", default='/data/pdb',
                        help='Path to PDB database files (default /data/pdb)')
    parser.add_argument("--compinchi",
                        default='http://ligand-expo.rcsb.org/' +
                        'dictionaries',
                        help='URL to download Components-inchi.ich' +
                             ' file for' +
                             'task stage dataimport '
                             '(default http://ligand-expo.rcsb.org/)')
    parser.add_argument("--pdbfileurl",
                        default='http://www.wwpdb.org/files',
                        help='URL to download ' +
                             DataImportTask.NONPOLYMER_TSV + ', ' +
                             DataImportTask.SEQUENCE_TSV + ', and ' +
                             DataImportTask.CRYSTALPH_TSV +
                             ' files for task stage dataimport '
                             '(default http://www.wwpdb.org/files)')
    parser.add_argument("--makeblastdb", default='makeblastdb',
                        help='Path to NCBI Blast makeblastdb program '
                             'ie /usr/bin/makeblastdb (default makeblastdb)')
    parser.add_argument("--pdbsequrl",
                        default='ftp://ftp.rcsb.org/pub/pdb/derived_data/'
                                'pdb_seqres.txt.gz',
                        help='ftp url to download rcsb sequences file '
                             '(default '
                             'ftp://ftp.rcsb.org/pub/pdb/derived_data/'
                             'pdb_seqres.txt.gz')
    parser.add_argument("--rdkitpython",
                        default='',
                        help='Path to directory containing python with '
                             'new rdkit installed')
    parser.add_argument("--log", dest="loglevel", choices=['DEBUG',
                        'INFO', DEFAULT_LOG_LEVEL, 'ERROR', 'CRITICAL'],
                        help="Set the logging level (default " +
                             DEFAULT_LOG_LEVEL + ")",
                        default=DEFAULT_LOG_LEVEL)
    parser.add_argument('--smtp', dest='smtp', help='Sets smtpserver to use '
                                                    '(default localhost)',
                        default='localhost')
    parser.add_argument('--smtpport', dest='smtpport',
                        help='Sets smtp server port (default 25)',
                        default='25')
    parser.add_argument('--skipimportwait', default=False, action='store_true',
                        help='Normally the import stage will wait if any of '
                             'the tsv files have not been updated since the'
                             'start of the current celpp week.  Setting this '
                             'flag bypasses this delay and downloads the'
                             'files')
    parser.add_argument('--importsleep', default=600, type=int,
                        help='Number of seconds to wait before re-checking '
                             'tsv files to see if they have been updated in '
                             'the import stage (default 600)')
    parser.add_argument('--importretry', default=60, type=int,
                        help='Number of times import stage should check if '
                             'tsv files have been updated (default 60, or 10 '
                             'hours with default --importsleep set to 600 '
                             'seconds)')
    parser.add_argument('--ftpconfig', dest='ftpconfig', help='File containing'
                        ' configuration to connect to ftp server.  If set,'
                        ' data from stages run during this invocation will be'
                        ' uploaded after the stage completes.  Format is same'
                        ' as ncftp config files with two added fields (' +
                        FtpFileTransfer.PATH + ', ' +
                        FtpFileTransfer.CHALLENGEPATH + ', ' +
                        FtpFileTransfer.SUBMISSIONPATH + ') ' +
                        'SEE: description of challengedata '
                        'stage above for example file')
    parser.add_argument('--replyto', dest='replytoaddress', default=None,
                        help='Email address to set as reply-to for emails to '
                             'external submitters')
    parser.add_argument('--version', action='version',
                        version=('%(prog)s ' + d3r.__version__))
    return parser.parse_args(args, namespace=parsed_arguments)


def main(args):
    """Main entry into celpprunner
    :param args: should be set to sys.argv which is a list of arguments
                 starting with script name as the first argument
    """
    blasttask = BlastNFilterTask('', p)
    dataimport = DataImportTask('', p)
    challenge = ChallengeDataTask('', p)
    glide = GlideTask('', p)
    makedb = MakeBlastDBTask('', p)
    prot = ProteinLigPrepTask('', p)
    vina = AutoDockVinaTask('', p)
    chimeraprep = ChimeraProteinLigPrepTask('', p)
    postevalstage = PostEvaluationTask('', p)
    desc = """
              Version {version}

              Runs the 11 stages (makedb, import, blast, challengedata,
              proteinligprep, {chimeraprep}, extsubmission, glide, vina,
              evaluation, & {postevaluation}) of CELPP processing pipeline
              (http://www.drugdesigndata.org)

              CELPP processing pipeline relies on a set of directories
              with specific structure. The pipeline runs a set of stages
              Each stage has a numerical value and a name. The numerical
              value denotes order and the stage name identifies separate
              tasks to run in the stage.

              The filesystem structure of the stage is:

              stage.<stage number>.<task name>

              The stage(s) run are defined via the required {stageflag} flag.

              To run multiple stages serially just pass a comma delimited
              list to the {stageflag} flag. Example: {stageflag} import,blast

              NOTE:  When running multiple stages serially the program will
                     not run subsequent stages if a task in a stage fails.
                     Also note order matters, ie putting blast,import will
                     cause celpprunner.py to run blast stage first.

              This program drops a pid lockfile
              (celpprunner.<stage>.lockpid) in celppdir to prevent duplicate
              invocation.

              When run, this program will examine the stage and see
              if work can be done.  If stage is complete or previous
              steps have not completed, the program will exit silently.
              If previous steps have failed or current stage already
              exists in an error or uncomplete state then program will
              report the error via email using addresses set in --email
              flag. Errors will also be reported via stderr/stdout.
              The program will also exit with nonzero exit code.

              This program utilizes simple token files to denote stage
              completion.  If within the stage directory there is a:

              '{complete}' file - then stage is done and no other
                                checking is done.

              'error' file - then stage failed.

              'start' file - then stage is running.

              Notification of stage start and end will be sent to
              addresses set via --email flag.

              Unless --customweekdir is set, this program will
              examine the 'celppdir' (last argument passed on
              commandline) to find the latest directory with this path:

              <year>/dataset.week.#

              The program will find the latest <year> and within
              that year the dataset.week.# with highest #.  The output
              directories created will be put within this directory.

              Setting --customweekdir will cause program to use 'celppdir'
              path.

              Setting the --createweekdir flag will instruct this
              program to create a new directory for the current
              celpp week/year before running any stage processing.

              NOTE: CELPP weeks start on Friday and end on Thursday
                    and week # follows ISO8601 rules so week numbers
                    at the end and start of the year are a bit
                    wonky.

              Breakdown of behavior of program is defined by
              value passed with {stageflag} flag:

              If {stageflag} '{createchallenge}'

              This is NOT a stage, but has the same effect as
              calling {stageflag} makedb,import,blast,challengedata
              The four stages that need to run to generate the challenge
              data package.

              If {stageflag} 'makedb'

              In this stage the file {pdb_seqres} is downloaded from
              an ftp site set by --pdbsequrl.
              This file is then gunzipped and NCBI makeblastdb
              (set by --makeblastdb) is run on it to create a blast
              database.  The files are stored in {makeblastdb_dirname}

              If {stageflag} 'import'

              In this stage 4 files are downloaded from urls specified
              by --compinchi and --pdbfileurl flags on the commandline
              into {dataimport_dirname} directory.

              The tsv files are (--pdbfileurl flag sets url to
              download these files from):

              {nonpolymer_tsv}
              {sequence_tsv}
              {crystal_tsv}

              The Components ich file is (--compinchi flag sets base url to
              download this file from):

              {compinchi_ich}

              This stage will just wait and retry if any of the tsv files
              have NOT been updated since the start of the current
              celpp week as determined by a HEAD request. To bypass
              this delay add --skipimportwait flag.  --importsleep denotes
              the time to wait before re-examining the update time of the
              tsv files and --importretry sets number of times to retry
              before giving up.

              If {stageflag} 'blast'

              Verifies {dataimport_dirname} exists and has '{complete}'
              file.  Also verifies {makeblastdb_dirname} exists and has
              '{complete}' file.  If both conditions are met then the
              'blast' stage is run which invokes script set by
              --blastnfilter flag and output stored in
              {blast_dirname}.
              Requires --pdbdb to be set to a directory with valid PDB
              database files.

              Note: --blastnfilter script is killed after time set with
              --blastnfiltertimeout flag.


              If {stageflag} 'challengedata'

              Verifies {blast_dirname} exists and has '{complete}'
              file.  If complete, this stage runs which invokes program
              set in --genchallenge flag to create a challenge dataset
              file.  The --pdbdb flag must also be set when calling this
              stage. If --ftpconfig is set with {challengepath} field then
              this stage will also upload the challenge dataset tarfile
              to the ftp server with path set by {challengepath}.  The
              code will also upload a {latest_txt} file containing name
              of the tarfile to the same destination overwriting any
              {latest_txt} file that already exists.

              Example file for --ftpconfig:

              {host} some.ftp.com
              {user} bob
              {passn} mypass
              {path} /celpp
              {challengepath} /challenge
              {submissionpath} /submissions


              If {stageflag} '{chimeraprep}'

              Verifies {challenge_dirname} exists and has '{complete}'
              file.  If complete, this stage runs which invokes program
              set in --chimeraprep flag to prepare pdb and inchi files
              storing output in {chimeraprep_dirname}.  --pdbdb flag
              must also be set when calling this stage.

              If {stageflag} 'proteinligprep'

              Verifies {challenge_dirname} exists and has '{complete}'
              file.  If complete, this stage runs which invokes program
              set in --proteinligprep flag to prepare pdb and inchi files
              storing output in {proteinligprep_dirname}.  --pdbdb flag
              must also be set when calling this stage.

              If {stageflag} 'extsubmission'

              Connects to server specified by --ftpconfig and downloads
              external docking submissions from {submissionpath} on remote
              server.

              Submissions should be named:

              celpp_weekXX_YYYY_dockedresults_ZZZZ.tar.gz as documented here:

              https://github.com/drugdata/d3r/wiki/Proposed-challenge-docked\
              -results-file-structure

              For each submission a directory named stage.X.ZZZZ.extsubmission
              will be created and uncompressed contents of package will be
              stored in that directory.  If data does not conform properly
              'error' file will be placed in directory denoting failure

              If {stageflag} 'glide'

              Verifies {proteinligprep_dirname} exists and has a '{complete}'
              file within it.  If complete, this stage runs which invokes
              program set in --glide flag to perform docking via glide
              storing output in {glide_dirname}

              If {stageflag} 'vina'

              Verifies {proteinligprep_dirname} exists and has a '{complete}'
              file within it.  If complete, this stage runs which invokes
              program set in --vina flag to perform docking via AutoDock Vina
              storing output in {vina_dirname}

              If {stageflag} 'evaluation'

              Finds all stage.{dockstage}.<algo> directories with '{complete}'
              files in them which do not end in name '{webdata}' and runs
              script set via --evaluation parameter storing the result of
              the script into stage.{evalstage}.<algo>.evaluation. --pdbdb flag
              must also be set when calling this stage.

              If {stageflag} '{postevaluation}'

              Finds all stage.{evalstage}.<algo>.evaluation directories and
               runs script set via --postevaluation parameter storing a
              summary of found docking evaluations into
              {postevalstage} into a file named summary.txt and a set
              of files with name Overall_RMSD_<candidate_type>.csv.
              In addition, results will be emailed to people in
              --summaryemail and --email lists.

              """.format(makeblastdb_dirname=makedb.get_dir_name(),
                         dataimport_dirname=dataimport.get_dir_name(),
                         blast_dirname=blasttask.get_dir_name(),
                         challenge_dirname=challenge.get_dir_name(),
                         createchallenge=CREATE_CHALLENGE,
                         proteinligprep_dirname=prot.get_dir_name(),
                         glide_dirname=glide.get_dir_name(),
                         vina_dirname=vina.get_dir_name(),
                         dockstage=str(glide.get_stage()),
                         evalstage=str(glide.get_stage() + 1),
                         complete=blasttask.COMPLETE_FILE,
                         chimeraprep_dirname=chimeraprep.get_dir_name(),
                         chimeraprep=CHIMERA_PREP,
                         compinchi_ich=DataImportTask.COMPINCHI_ICH,
                         pdb_seqres=MakeBlastDBTask.PDB_SEQRES_TXT_GZ,
                         nonpolymer_tsv=DataImportTask.NONPOLYMER_TSV,
                         sequence_tsv=DataImportTask.SEQUENCE_TSV,
                         crystal_tsv=DataImportTask.CRYSTALPH_TSV,
                         webdata=EvaluationTaskFactory.WEB_DATA_SUFFIX,
                         latest_txt=ChallengeDataTask.LATEST_TXT,
                         host=FtpFileTransfer.HOST,
                         user=FtpFileTransfer.USER,
                         passn=FtpFileTransfer.PASS,
                         path=FtpFileTransfer.PATH,
                         challengepath=FtpFileTransfer.CHALLENGEPATH,
                         submissionpath=FtpFileTransfer.SUBMISSIONPATH,
                         version=d3r.__version__,
                         stageflag=STAGE_FLAG,
                         postevaluation=POST_EVAL,
                         postevalstage=postevalstage.get_dir_name())

    theargs = _parse_arguments(desc, args[1:])
    theargs.program = args[0]
    theargs.version = d3r.__version__

    util.setup_logging(theargs)

    try:
        return run_stages(theargs)
    except Exception:
        logger.exception("Error caught exception")
        return 2


if __name__ == '__main__':  # pragma: no cover
    sys.exit(main(sys.argv))



__author__ = 'robswift'
__project__ = 'blastnfilter'

import sys
import os
import textwrap

def print_usage():
    w = textwrap.TextWrapper()
    w.expand_tabs = False
    try:
        rows, columns = os.popen('stty size', 'r').read().split()
        w.width = int(columns)
    except ValueError:
        pass
    w.initial_indent = ''
    w.subsequent_indent = ''

    usage = "usage: blastnfilter --nonpolymertsv --sequencetsv --crystalpH --pdbblastdb --pdbdb --compinchi --outdir"
    print
    print w.fill(usage)
    print

def interface(args):
    flags = ('--nonpolymertsv', '--sequencetsv', '--crystalpH', '--pdbblastdb', '--pdbdb', '--compinchi', '--outdir',)

    if len(args) == 0:
        print "\nNo arguments specified"
        print_usage()
        sys.exit(1)

    for flag in [x for x in args if '--' in x]:
        if flag not in flags:
            print "\n Unrecognized input flag: %s\n" % flag
            print_usage()
            sys.exit(1)

    parser = ParseArgs(args)
    return parser

class ParseArgs:
    def __init__(self, args):
        self.args = args
        self.required = ('--nonpolymertsv', '--sequencetsv', '--crystalpH', '--pdbblastdb', '--pdbdb', '--compinchi',
                         '--outdir',)

    def get_string(self, input_string):
        if input_string in self.required:
            try:
                index = self.args.index(input_string) + 1
            except ValueError:
                # the flag wasn't set, so exit
                print "\n %s is required" % input_string
                print_usage()
                sys.exit(1)
            # the flag was set, so check if a value was set, otherwise exit
            try:
                if self.args[index] in self.required:
                    print "\n %s was set but a value was not specified" % input_string
                    print_usage()
                    sys.exit(1)
            except IndexError:
                print "\n %s was set but a value was not specified" % input_string
                print_usage()
                sys.exit(1)
            # return the value that was set
            return format(self.args[index])

class SplitInput:
    def __init__(self, itf):
        self.non_polymer = itf.get_string('--nonpolymertsv')
        self.polymer = itf.get_string('--sequencetsv')
        self.ph = itf.get_string('--crystalpH')
        self.blast_db = itf.get_string('--pdbblastdb')
        self.pdb_path = itf.get_string('--pdbdb')
        self.compinchi = itf.get_string('--compinchi')
        self.out = itf.get_string('--outdir')





#!/usr/bin/env python

import commands
import os
import glob
import logging
import time
import re 

logger = logging.getLogger()
logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level   = logging.DEBUG )

#challenge_data_path = "/data/celpp/2015/dataset.week.47/stage.2.blastnfilter"
#pdb_protein_location = "/data/pdb.extracted"
#full_copy_location = ""
#target_copy_location = ""

# Name for python binary
PYTHON_BINARY_NAME = 'python'

rdkit_smiles_to_3d_sdf_text = '''
import rdkit.Chem
import rdkit.Chem.AllChem
import sys

if not(len(sys.argv)) == 3:
    print "python smiles2Mol.py inputSmiles outputSdf"
    sys.exit()

smiles = open(sys.argv[1]).read().strip()
mol = rdkit.Chem.MolFromSmiles(smiles)
molH = rdkit.Chem.AddHs(mol)
rdkit.Chem.AllChem.EmbedMolecule(molH)
rdkit.Chem.AllChem.UFFOptimizeMolecule(molH)

w = rdkit.Chem.SDWriter(sys.argv[2])
w.write(molH)
w.close()
'''


chimera_prep_text = '''
import chimera
import sys
opened = chimera.openModels.open(sys.argv[1])
mol = opened[0]

import DockPrep

DockPrep.prep([mol])
from WriteMol2 import writeMol2
with open(sys.argv[2],'wb') as of:
    writeMol2([mol], of)
'''

def extract_info_from_s2(stage_2_out):
    info_dic = {}
    f = open(stage_2_out,"r")
    lines = f.readlines()
    f.close()
    for line in lines:
        #print line.split(",")[0]
        info_title = line.split(",")[0]
        info_value_1 = line.split(",")[1] 
        if line.split(",")[0] == "inchi":
            info_title = line.split(", InChI=")[0] 
            info_value_1 = line.split(", InChI=")[1].split("\n")[0]
        else:
            try:
                info_value_2 = line.split(",")[2].split("\n")[0].strip()
                info_title = line.split(",")[0]
                info_value_1 = line.split(",")[1].strip() 
            except:
                info_value_2 = False
                info_title = line.split(",")[0]
                info_value_1 = line.split(",")[1].split("\n")[0].strip()
                pass
        #if not 
        if not info_value_2:
            #just use the first item if there are multiple entries (use the highest resolution one)
            if info_title not in info_dic:
                info_dic[info_title] = info_value_1
        else:
            if info_title not in info_dic:
                info_dic[info_title] = (info_value_1,info_value_2) 
    return info_dic

#copy all the txt files from the output stage 2 location and create folder for each of this named by the query entry
#check if it finished later needed 

def ligand_prepare(ligand_smile, out_lig_file, rdkit_python):
#    commands.getoutput("$SCHRODINGER/ligprep -WAIT -i 0 -nt -s 1 -g -ismi %s -omae %s"%(ligand_smile, out_lig_file) ) 
#    return os.path.isfile(out_lig_file)
    if os.path.isfile(out_lig_file):
        logging.info('Ligand file %s is already prepared. Skipping.' %(out_lig_file))
        return True

    # Prepare a 3D version of the ligand using babel
    unprep_lig_file_1 = ligand_smile.replace('.smi','_unprep_step1.sdf')
    with open('rdkit_smiles_to_3d_sdf.py','wb') as of:
        of.write(rdkit_smiles_to_3d_sdf_text)

    rdkitpythonpath = os.path.join(rdkit_python, PYTHON_BINARY_NAME)
    commands.getoutput(rdkitpythonpath + ' rdkit_smiles_to_3d_sdf.py ' +
                       ligand_smile + ' ' + unprep_lig_file_1 +
                       ' > rdkit_smiles_to_3d_sdf_out 2>&1')
    unprep_lig_file_2 = ligand_smile.replace('.smi','_unprep_step2.mol2')
    commands.getoutput('babel -isdf %s -omol2 %s' %(unprep_lig_file_1, unprep_lig_file_2))
    #unprep_lig_file = ligand_smile.replace('.smi','_unprep.mol2')
    #commands.getoutput('babel -ismi %s -omol2 %s --gen3D' %(ligand_smile,unprep_lig_file))
    with open('chimeraPrep.py','wb') as of:
        of.write(chimera_prep_text) 
    commands.getoutput('chimera --nogui --script "chimeraPrep.py %s %s" >& chimeraLigPrep.out' %(unprep_lig_file_2, out_lig_file))
    #time.sleep(sleep_time) 
    return os.path.isfile(out_lig_file)

def split_complex (part, complex_file, out_split):
    commands.getoutput("$SCHRODINGER/run split_structure.py -many_files -m %s %s %s"%(part, complex_file, out_split))
    out_receptor = os.path.splitext(out_split)[0] + "_receptor1"+ os.path.splitext(out_split)[1]
    if os.path.isfile(out_receptor):
        return out_receptor 
    else:
        return False

def prepare_protein (protein_file, prepared_protein, sleep_time = 300):
    #here the prepared protein should has ".mol2" as extension
    #commands.getoutput("$SCHRODINGER/utilities/prepwizard %s %s"%(protein_file, prepared_protein))
    with open('chimeraPrep.py','wb') as of:
        of.write(chimera_prep_text) 
    commands.getoutput('chimera --nogui --script "chimeraPrep.py %s %s" >& chimeraProtPrep.out' %(protein_file, prepared_protein))
    #time.sleep(sleep_time) 
    if os.path.isfile(prepared_protein):
        return True
    else:
        return False


def main_proteinprep (challenge_data_path, pdb_protein_path, working_folder, rdkit_python ):
    os.chdir(working_folder)
    current_dir_layer_1 = os.getcwd()

    ## Get all potential target directories and candidates within
    valid_candidates = {}
    #target_ligands = {}
    pot_target_dirs = list(os.walk(challenge_data_path))[0][1]
    #target_ids = []
    # Ensure that the directories are valid
    for pot_target_id in pot_target_dirs:
        os.chdir(current_dir_layer_1)
        # Does it look like a pdb id?
        if len(pot_target_id) != 4:
            logging.info('Filtering potential target directories: %s is not 4 characters long. Skipping' %(pot_target_id))
            continue
        commands.getoutput('mkdir %s' %(pot_target_id))
        #target_ids.append(pot_target_dir)
        valid_candidates[pot_target_id] = []
        target_dir_path = os.path.join(challenge_data_path, pot_target_id)

        # Pull in the ligand inchi
        lig_smiles_files = glob.glob('%s/lig_*.smi' %(target_dir_path))
        if len(lig_smiles_files) != 1:
            logging.info('Unable to find unambiguous ligand smiles for %s - glob returned %r' %(pot_target_id, lig_smiles_files))
            continue
        lig_smiles_file = lig_smiles_files[0]
        local_smiles_file = os.path.basename(lig_smiles_file)
        dest_smiles_file = os.path.join(pot_target_id, local_smiles_file)
        commands.getoutput('cp %s %s' %(lig_smiles_file, dest_smiles_file))
        center_file = os.path.join(target_dir_path,'center.txt')
        commands.getoutput('cp %s %s' %(center_file, pot_target_id))
        ## Get the ligand center of mass for the "LMCSS" candidate (all of the other candidates have been aligned to this one)
        LMCSS_ligand_filenames = glob.glob('%s/LMCSS-*-lig.pdb'%(target_dir_path))
        if len(LMCSS_ligand_filenames) != 1:
            logging.info("Failed to find LMCSS structure's ligand file. There should be one match but I found %r" %(LMCSS_ligand_filenames))
        LMCSS_ligand_filename = LMCSS_ligand_filenames[0]
        
        # Copy in each valid candidate
        for candidate_file in glob.glob('%s/*-%s_*.pdb' %(target_dir_path, pot_target_id)):
            # The LMCSS ligand will be in a pdb file called something like celpp_week19_2016/1fcz/LMCSS-1fcz_1fcz-156-lig.pdb
            # We want to make sure we don't treat this like a receptor
            if 'lig.pdb' in candidate_file:
                continue
            commands.getoutput('cp %s %s' %(candidate_file, pot_target_id))
            candidate_local_file = os.path.basename(candidate_file)
            valid_candidates[pot_target_id].append((local_smiles_file, candidate_local_file))
                
    for target_id in valid_candidates.keys():
        os.chdir(target_id)

        ######################
        #step 6, prepare all proteins
        ######################
        for smiles_filename, candidate_filename in valid_candidates[target_id]:
            ## Parse the candidate name 
            ## Get the method type, target, and candidate info from the filename
            # for example, this will parse 'hiResApo-5hib_2eb2_docked.mol' into [('hiResApo', '5hib', '2eb2')]
            
            parsed_name = re.findall('([a-zA-Z0-9]+)-([a-zA-Z0-9]+)_([a-zA-Z0-9]+)-?([a-zA-Z0-9]*).pdb', candidate_filename)
            if len(parsed_name) != 1:
                logging.info('Failed to parse docked structure name "%s". Parsing yielded %r' %(candidate_filename, parsed_name))
                continue
            candidate_structure_type = parsed_name[0][0]
            candidate_structure_target = parsed_name[0][1]
            candidate_structure_candidate = parsed_name[0][2]
            candidate_structure_ligand = parsed_name[0][2]
            
            # Prepare the ligand
            if not ligand_prepare(smiles_filename, smiles_filename.replace('.smi','_prepared.mol2'), rdkit_python):
                logging.info("Unable to prepare the ligand for this query protein:%s"%target_id)
                #os.chdir(current_dir_layer_1)
                continue 

            # Split the complex 
            #candidate_prefix = candidate_filename.replace('.pdb','')
            candidate_prefix = '%s-%s_%s' %(candidate_structure_type,
                                            candidate_structure_target,
                                            candidate_structure_candidate)
            
            out_split = candidate_prefix+ "_split.pdb"
            out_receptor = split_complex("pdb", candidate_filename, out_split)
            
            
            if not out_receptor:
                logging.info("Unable to split this protein:%s"%(candidate_filename))
                #os.chdir(current_dir_layer_1)
                continue

            logging.info("Successfully split this protein:%s, go to preparation step"%(candidate_filename))
            prepared_protein_mol2 = candidate_prefix + "_prepared.mol2"
            #pass the wizard sleep time here
            preparation_result = prepare_protein(out_receptor,prepared_protein_mol2, 180 )
            if not preparation_result:
                logging.info("Unable to prepare this protein:%s"%(out_split))
                #os.chdir(current_dir_layer_1)
                continue
            #convert into pdb format
            out_prepare_pdb = candidate_prefix + "_prepared.pdb"
            commands.getoutput("$SCHRODINGER/utilities/pdbconvert -imae %s -opdb %s"%(prepared_protein_mol2, out_prepare_pdb))
            logging.info("Successfully prepared this protein:%s"%(out_prepare_pdb))
        os.chdir(current_dir_layer_1)
                        
                

if ("__main__") == (__name__):
    from argparse import ArgumentParser
    parser = ArgumentParser()
    parser.add_argument("-p", "--pdbdb", metavar = "PATH", help = "PDB DATABANK which we will dock into")
    parser.add_argument("-c", "--candidatedir", metavar="PATH", help = "PATH where we could find the stage 2 output")
    parser.add_argument("-o", "--outdir", metavar = "PATH", help = "PATH where we run stage 3")
    parser.add_argument("-r", "--rdkitpython", metavar="PATH",
                        help="Path for python build with new "
                             "version of rdkit.",
                        default="")
    #parser.add_option("-s", "--sleep", metavar = "VALUE", help = "Sleep time for protein prep")
    #parser.add_option("-u", "--update", default = False, action = "store_true", help = "update the protein generation and docking step")
    logger = logging.getLogger()
    logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level   = logging.DEBUG )
    opt = parser.parse_args()
    pdb_location = opt.pdbdb
    challenge_data_path = opt.candidatedir
    result_path = opt.outdir
    rdkit_python = opt.rdkitpython
    #sleep_time = opt.sleep
    #running under this dir
    running_dir = os.getcwd()
    main_proteinprep(challenge_data_path, pdb_location, result_path, rdkit_python)
    #move the final log file to the result dir
    log_file_path = os.path.join(running_dir, 'final.log')
    commands.getoutput("mv %s %s"%(log_file_path, result_path))




#!/usr/bin/env python

import argparse
import pickle
import logging
import os
import re
import sys

__author__ = 'sliu'

# pass 1, where contains stage 7 folders, 2,
# contain the stage 8 result, 4 contain genchallenge data final files

RMSD_PICKLE = 'RMSD.pickle'
FINAL_LOG = 'final.log'
CHALL_FINAL_LOG = FINAL_LOG
LMCSS = 'LMCSS'
SMCSS = 'SMCSS'
HI_TANIMOTO = 'hiTanimoto'
HI_RESAPO = 'hiResApo'
HI_RESHOLO = 'hiResHolo'
SUMMARY_TXT = 'summary.txt'
OVERALL_RMSD_PREFIX = 'Overall_RMSD_'
CSV_SUFFIX = '.csv'

logger = logging.getLogger()


def check_case_number(log_file, phrase):
    """Counts number of occurrences of `phrase`
       in file path passed in.
    :param log_file: path to file to examine
    :param phrase: phrase to search for in each line of file
    :returns: count of occurrences as int upon success, -1 if
              there was an error or if `log_file` or 1phrase`
              is None
    """

    total_number = -1
    if log_file is None:
        logger.error('log file is None')
        return total_number

    if phrase is None:
        logger.error('phrase is None')
        return total_number

    log_f = None
    try:
        log_f = open(log_file, "r")
        total_number = 0
        for log_line in log_f:
            if phrase in log_line:
                total_number += 1
    except IOError:
        logger.exception('Caught exception attempting to load file: ' +
                         log_file)
    finally:
        if log_f is not None:
            log_f.close()
    return total_number


def get_dock_scores_as_list(pickle_file, ctype=LMCSS):
    """Get list of dock scores for candidate type `ctype`
       from pickle_file `pickle_file`
       The format of the pickle file should be
       { 'ligand': {'ctype': score}}
    :param pickle_file: pickle file to parse
    :param ctype: candidate type
    :returns: list of values, no checking on types could be strings
              ints, floats etc.
    """
    dock_scores = []
    if pickle_file is None:
        logger.error('Pickle file is None')
        return dock_scores

    if ctype is None:
        logger.error('Candidate Type is None')
        return dock_scores

    try:
        p_f = open(pickle_file, "rb")
        p_d = pickle.load(p_f)
        p_f.close()
        data = []
        for ligand in p_d:
            try:
                value = p_d[ligand][ctype]
                dock_scores.append(value)
            except KeyError:
                logger.error('No ' + ctype +
                             ' for ' + ligand +
                             ' skipping...')
                continue
        return dock_scores
    except Exception:
        logger.exception('Caught Exception just returning empty list')
        return []


def get_list_of_stats(list_of_dock_scores):
    """Extract average for given type
       by getting scores for all ligands with scores
       of type `ctype` in `pickle_file`
       The format of the pickle file should be
       { 'ligand': {'ctype': score}}
    :param list_of_dock_scores: list of dock scores
    :returns: tuple (count, min, max, average)
    """
    if list_of_dock_scores is None or len(list_of_dock_scores) is 0:
        logger.error('Unable to get and dock scores')
        return -1, -1, -1, -1
    try:
        count = len(list_of_dock_scores)
        average = sum(list_of_dock_scores)/count
        return (count,
                min(list_of_dock_scores),
                max(list_of_dock_scores),
                average)
    except Exception:
        logger.exception('Caught Exception just returning -1, -1')
        return -1, -1, -1, -1


def _get_pickle_paths(path_list):
    """Given a list of evaluation task directories
       find all pickle files and return that as
       a list as well as a count of those that failed
    """
    pickle_list = []
    no_pickle_count = 0

    if path_list is None:
        logger.error('Path list is None')
        return pickle_list, no_pickle_count

    logger.info('Given ' + str(len(path_list)) + ' paths to look for'
                                                 ' pickle files in')

    for all_stage_7 in path_list:
        full_path = os.path.join(all_stage_7, RMSD_PICKLE)
        if os.path.isfile(full_path):
            pickle_list.append(full_path)
        else:
            logging.info("The pickle file " + full_path +
                         "does not exist")
            no_pickle_count += 1

    return pickle_list, no_pickle_count


def _get_submission_name_from_pickle_path(path, prefix, suffix):
    """Given path to pickle path in evaluation task
       get the submission name which is basically
       the prefix and suffix values lopped off
    """
    dir_name = os.path.dirname(path)
    base_name = os.path.basename(dir_name)
    sub_name_suffix = re.sub('^' + prefix, '', base_name)
    return re.sub(suffix, '', sub_name_suffix)


def generate_overall_csv(evaluation_path, challenge_dir, post_evaluation_path,
                         candidates_type=LMCSS,
                         eval_stage_prefix='',
                         eval_suffix=''):

    not_valid_pickle = 0

    all_pickle_files, non_pickle_case = _get_pickle_paths(evaluation_path)

    candidates_report = os.path.join(challenge_dir, CHALL_FINAL_LOG)
    total_candidates = check_case_number(candidates_report,
                                         "Succsessfully generate "
                                         "this protein:" + candidates_type)

    overall_csv = open(os.path.join(post_evaluation_path,
                                    OVERALL_RMSD_PREFIX + candidates_type +
                                    CSV_SUFFIX), 'w')

    summary_txt = open(os.path.join(post_evaluation_path,
                                    SUMMARY_TXT), 'a')

    s_line_format = '%-25s%-12s%-12s%-12s%-12s\n'
    summary_txt.write(s_line_format % (candidates_type + ' (' +
                                       str(total_candidates) +
                                       ' dockable)',
                                       '#Docked',
                                       'Min RMSD',
                                       'Max RMSD',
                                       'Mean RMSD'))

    summary_txt.write('-'*72 + '\n')

    data_line_format = '%s,%s,%s,%s,%s,%s\n'
    full_data_lines = [data_line_format % ("SubmissionID for " +
                                           candidates_type,
                                           "# Docked",
                                           "# Dockable",
                                           "Min RMSD",
                                           "Max RMSD",
                                           "Mean RMSD")]
    for p_file in all_pickle_files:
        sub_name = _get_submission_name_from_pickle_path(p_file,
                                                         eval_stage_prefix,
                                                         eval_suffix)
        try:
            d_scores = get_dock_scores_as_list(p_file,
                                               ctype=candidates_type)
            num_docked, minrmsd, maxrmsd, avgrmsd = get_list_of_stats(d_scores)

            pc_docked = '(NA%)'
            if total_candidates > 0:
                percent_docked = float(num_docked) / float(total_candidates)
                percent_docked *= 100.0
                pc_docked = ' (%3.0f%%)' % percent_docked

            full_data_lines.append(data_line_format % (sub_name,
                                                       num_docked,
                                                       total_candidates,
                                                       '%.3f' % minrmsd,
                                                       '%.3f' % maxrmsd,
                                                       '%.3f' % avgrmsd))
            summary_txt.write(s_line_format % (sub_name,
                                               str(num_docked) + pc_docked,
                                               '%.3f' % minrmsd,
                                               '%.3f' % maxrmsd,
                                               '%.3f' % avgrmsd))
        except IOError:
            logging.exception('Error writing entry')
        except (ValueError, TypeError):
            full_data_lines.append(data_line_format % (sub_name,
                                                       "N/A",
                                                       total_candidates,
                                                       "N/A",
                                                       "N/A",
                                                       "N/A"))
            summary_txt.write(s_line_format % (sub_name,
                                               "N/A",
                                               "N/A",
                                               "N/A",
                                               "N/A"))
            logging.exception("This pickle file :%s do not have valid data for "
                              "candidates type  %s" % (p_file,
                                                       candidates_type))
            not_valid_pickle += 1

    logging.info("We got : %s cases without pickle files and %s cases have "
                 "invalid pickle files for "
                 "candidates type %s" % (non_pickle_case,
                                         not_valid_pickle,
                                         candidates_type))
    overall_csv.writelines(full_data_lines)
    overall_csv.close()

    summary_txt.write('\n\n')
    summary_txt.close()


def main(args):
    """Main entry point of script"""
    desc = """
              Summarizes results for all docking evaluations
              passed into script
    """
    help_formatter = argparse.RawDescriptionHelpFormatter
    parser = argparse.ArgumentParser(description=desc,
                                     formatter_class=help_formatter)
    parser.add_argument("outdir",
                        help="PATH where we will run the evaluate stage")
    parser.add_argument("-e", "--evaluationdir", metavar="PATH",
                        action='append',
                        help="PATH where we could find the evaluation "
                             "stage output")
    parser.add_argument("-c", "--challengedir", metavar="PATH",
                        help="PATH to celpp week directory under challenge "
                             "task ")
    parser.add_argument("--stageprefix", default='stage.7.',
                        help="Evaluation task dir prefix (default stage.7.) ")
    parser.add_argument("--evaluationsuffix",
                        default='.extsubmission.evaluation$|.evaluation$',
                        help="Evaluation task dir suffix regex"
                             "(default .extsubmission.evaluati"
                             "on$|.evaluation$)")

    opt = parser.parse_args(args[1:])
    logging.basicConfig(format='%(asctime)s: %(message)s',
                        datefmt='%m/%d/%y %I:%M:%S',
                        filename=os.path.join(opt.outdir, FINAL_LOG),
                        filemode='w', level=logging.INFO)
    # main calculation
    for candidate_type in [LMCSS, SMCSS, HI_TANIMOTO,
                           HI_RESAPO, HI_RESHOLO]:
        try:
            generate_overall_csv(opt.evaluationdir,
                                 opt.challengedir,
                                 opt.outdir,
                                 candidates_type=candidate_type,
                                 eval_stage_prefix=opt.stageprefix,
                                 eval_suffix=opt.evaluationsuffix)
        except Exception:
            logger.exception('Caught exception running generate_overall_csv '
                             'for candidate type ' + candidate_type)
            continue
    return 0


if __name__ == '__main__':  # pragma: no cover
    sys.exit(main(sys.argv))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_post_evaluation
----------------------------------

Tests for `post_evaluation` module.
"""

import unittest
import tempfile
import os
import os.path
import shutil
import pickle

from d3r import post_evaluation
from d3r.celpp.task import D3RParameters


class TestPostEvaluation(unittest.TestCase):
    """Tests post_evaluation commandline script
    """
    def setUp(self):
        pass

    def test_check_case_number(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # log file is none
            self.assertEqual(post_evaluation.check_case_number(None,
                                                               'hi'), -1)

            # non existant file
            noexist = os.path.join(temp_dir, 'doesnotexist.txt')
            self.assertEqual(post_evaluation.check_case_number(noexist,
                                                               'hi'), -1)

            # passing directory which should fail
            self.assertEqual(post_evaluation.check_case_number(temp_dir,
                                                               'hi'), -1)

            # empty file
            efile = os.path.join(temp_dir, 'emptyfile.txt')
            open(efile, 'a').close()
            self.assertEqual(post_evaluation.check_case_number(efile,
                                                               'hi'), 0)

            # phrase is None
            self.assertEqual(post_evaluation.check_case_number(efile,
                                                               None), -1)
            # file with couple matches
            tfile = os.path.join(temp_dir, 'hi.txt')
            f = open(tfile, 'w')
            f.write('Hi\n bye some phrase\n some\n'
                    ' phrase\ some phrase sdfsdf some phrase\n')
            f.flush()
            f.close()
            self.assertEqual(post_evaluation.check_case_number(tfile,
                                                               'some phrase'),
                             2)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_dock_scores_as_list(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # test passing dir as pickle file
            res = post_evaluation.get_dock_scores_as_list(temp_dir)
            self.assertEqual(res, [])

            # test None for Pickle file
            res = post_evaluation.get_dock_scores_as_list(None,
                                                          ctype=None)
            self.assertEqual(res, [])

            smcss = post_evaluation.SMCSS
            hi_tanimoto = post_evaluation.HI_TANIMOTO
            hi_resapo = post_evaluation.HI_RESAPO
            hi_resholo = post_evaluation.HI_RESHOLO
            apickle = os.path.join(temp_dir, 'apickle.pickle')
            data = {'5ka3': {smcss: 1.0, hi_resholo: 2.0, hi_tanimoto: 'foo',
                             post_evaluation.LMCSS: 4.0},
                    '5uud': {hi_resholo: 5.0, smcss: 6.0, hi_resapo: 7.0,
                             hi_tanimoto: 8.0, post_evaluation.LMCSS: 9.0}}
            f = open(apickle, 'w')
            pickle.dump(data, f)
            f.flush()
            f.close()

            # test None for ctype
            res = post_evaluation.get_dock_scores_as_list(apickle,
                                                               ctype=None)
            self.assertEqual(res, [])

            # test where no matches found
            res = post_evaluation.get_dock_scores_as_list(apickle,
                                                          ctype='foo')
            self.assertEqual(res, [])

            # test where 1 dock is found
            res = post_evaluation.get_dock_scores_as_list(apickle,
                                                          ctype=hi_resapo)
            self.assertEqual(res, [7.0])

            # test where 2 docked is found
            res = post_evaluation.get_dock_scores_as_list(apickle,
                                                          ctype=smcss)
            self.assertTrue(6.0 in res)
            self.assertTrue(1.0 in res)
            self.assertEqual(len(res), 2)

            # test default candidate type
            res = post_evaluation.get_dock_scores_as_list(apickle)
            self.assertTrue(4.0 in res)
            self.assertTrue(9.0 in res)
            self.assertEqual(len(res), 2)

            # test where score is not a number
            res = post_evaluation.get_dock_scores_as_list(apickle,
                                                          ctype=hi_tanimoto)
            self.assertTrue('foo' in res)
            self.assertTrue(8.0 in res)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_list_of_stats(self):
        res = post_evaluation.get_list_of_stats(None)
        self.assertEqual(res, (-1, -1, -1, -1))

        res = post_evaluation.get_list_of_stats([1.0])
        self.assertEqual(res, (1, 1.0, 1.0, 1.0))

        res = post_evaluation.get_list_of_stats([3.0, 1.0])
        self.assertEqual(res, (2, 1.0, 3.0, 2.0))

        res = post_evaluation.get_list_of_stats([1.0, 'foo'])
        self.assertEqual(res, (-1, -1, -1, -1))

        res = post_evaluation.get_list_of_stats([3.0, 1.0, 2.0, 14.0])
        self.assertEqual(res, (4, 1.0, 14.0, 5.0))

    def test_get_pickle_paths(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # none passed in
            p_list, count = post_evaluation._get_pickle_paths(None)
            self.assertEqual(p_list, [])
            self.assertEqual(count, 0)

            # emptylist
            p_list, count = post_evaluation._get_pickle_paths([])
            self.assertEqual(p_list, [])
            self.assertEqual(count, 0)

            # one path no pickle
            p_list, count = post_evaluation._get_pickle_paths([temp_dir])
            self.assertEqual(p_list, [])
            self.assertEqual(count, 1)

            # one path with pickle
            pfile = os.path.join(temp_dir, post_evaluation.RMSD_PICKLE)
            open(pfile, 'a').close()
            p_list, count = post_evaluation._get_pickle_paths([temp_dir])
            self.assertEqual(len(p_list), 1)
            self.assertTrue(pfile in p_list)
            self.assertEqual(count, 0)

            # two paths and only one with pickle
            subdir = os.path.join(temp_dir, 'foo')
            os.makedirs(subdir, mode=0o775)
            p_list, count = post_evaluation._get_pickle_paths([temp_dir,
                                                               subdir])
            self.assertEqual(len(p_list), 1)
            self.assertTrue(pfile in p_list)
            self.assertEqual(count, 1)

            # two paths with pickles
            p2file = os.path.join(subdir, post_evaluation.RMSD_PICKLE)
            open(p2file, 'a').close()

            p_list, count = post_evaluation._get_pickle_paths([subdir,
                                                               temp_dir])
            self.assertEqual(len(p_list), 2)
            self.assertTrue(pfile in p_list)
            self.assertTrue(p2file in p_list)
            self.assertEqual(count, 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_submission_name_from_pickle_path(self):

        somepath = os.path.join('data', 'celpp', '2017', 'dataset.week.10',
                                'stage.7.foo_dock.extsubmission.evaluation',
                                post_evaluation.RMSD_PICKLE)
        prefix = 'stage.7.'
        suffix = '.extsubmission.evaluation$|.evaluation$'
        name = post_evaluation._get_submission_name_from_pickle_path(somepath,
                                                                     prefix,
                                                                     suffix)
        self.assertEqual(name, 'foo_dock')

        # non external submission
        somepath = os.path.join('data', 'celpp', '2017', 'dataset.week.10',
                                'stage.7.foo_dock.evaluation',
                                post_evaluation.RMSD_PICKLE)

        self.assertEqual(name, 'foo_dock')

    def test_generate_overall_csv_valid_single_evaldataset(self):
        temp_dir = tempfile.mkdtemp()
        try:
            result_dir = os.path.join(temp_dir, 'result')
            os.makedirs(result_dir, mode=0o755)

            chall_dir = os.path.join(temp_dir, 'challdir')
            os.makedirs(chall_dir, mode=0o755)

            # write out the final.log file
            finalfile = os.path.join(chall_dir, post_evaluation.CHALL_FINAL_LOG)
            f = open(finalfile, 'w')
            f.write("""03/04/17 03:32:54: ============Start to work in this query protein: 5uub============
03/04/17 03:32:55: Warning: Found multiple ligand for this protein:LMCSS-5uub_5un3-XYP.pdb
03/04/17 03:32:55: Succsessfully generate this protein:LMCSS-5uub_5un3-XYP.pdb
03/04/17 03:32:55: Ligand center for this case:LMCSS-5uub_5un3-XYP-lig.pdb is   38.448,   73.584,   58.680
03/04/17 03:33:00: Succsessfully generate this protein:SMCSS-5uub_5fsj-OXY.pdb
03/04/17 03:33:12: ============Start to work in this query protein: 5ur3============
03/04/17 03:33:12: Succsessfully generate this protein:LMCSS-5ur3_3njq-NJQ.pdb
03/04/17 03:33:12: Ligand center for this case:LMCSS-5ur3_3njq-NJQ-lig.pdb is  -27.326,  -19.578,  -43.347
03/04/17 03:33:19: Succsessfully generate this protein:hiResHolo-5ur3_4p3h-25G.pdb
03/04/17 03:33:25: Succsessfully generate this protein:LMCSS-5uud_1qf2-TI3.pdb
03/04/17 03:33:25: Ligand center for this case:LMCSS-5uud_1qf2-TI3-lig.pdb is   38.308,   38.885,   -3.382
03/04/17 03:33:40: Succsessfully generate this protein:hiTanimoto-5uud_1gxw-VAL.pdb""")
            f.flush()
            f.close()

            task_dir = os.path.join(temp_dir,
                                    'stage.7.foo_dock.extsubmission.evaluation')
            os.makedirs(task_dir, mode=0o755)

            # write out the pickle dir
            smcss = post_evaluation.SMCSS
            hi_tanimoto = post_evaluation.HI_TANIMOTO
            hi_resapo = post_evaluation.HI_RESAPO
            hi_resholo = post_evaluation.HI_RESHOLO
            apickle = os.path.join(task_dir, post_evaluation.RMSD_PICKLE)
            data = {'5ka3': {smcss: 1.0, hi_resholo: 2.0, hi_tanimoto: 'foo',
                             post_evaluation.LMCSS: 4.0},
                    '5uud': {hi_resholo: 5.0, smcss: 6.0, hi_resapo: 7.0,
                             hi_tanimoto: 8.0, post_evaluation.LMCSS: 9.0}}
            f = open(apickle, 'w')
            pickle.dump(data, f)
            f.flush()
            f.close()
            post_evaluation.generate_overall_csv([task_dir], chall_dir,
                                                 result_dir,
                                                 post_evaluation.LMCSS)

            summary_file = os.path.join(result_dir,
                                        post_evaluation.SUMMARY_TXT)
            self.assertTrue(os.path.isfile(summary_file))
            f = open(summary_file, 'r')
            data = f.read()
            f.close()

            self.assertTrue(post_evaluation.LMCSS + ' (3 dockable)' in data)
            self.assertTrue(('stage.7.foo_dock.' +
                            'extsubmission.evaluation') in data)
            self.assertTrue('2 ( 67%)    4.000       9.000' in data)

            csv_file = os.path.join(result_dir,
                                    post_evaluation.OVERALL_RMSD_PREFIX +
                                    post_evaluation.LMCSS +
                                    post_evaluation.CSV_SUFFIX)
            self.assertTrue(os.path.isfile(csv_file))
            f = open(csv_file, 'r')
            data = f.read()
            f.close()
            self.assertTrue('SubmissionID for LMCSS,# Docked,' in data)
            self.assertTrue('stage.7.foo_dock.extsubmission' in data)
            self.assertTrue('submission.evaluation,2,3,4.000,9.000,6.500' in data)

            # test prefix suffix removal
            os.remove(summary_file)
            post_evaluation.generate_overall_csv([task_dir], chall_dir,
                                                 result_dir,
                                                 post_evaluation.LMCSS,
                                                 eval_stage_prefix='stage.7.',
                                                 eval_suffix='.extsubmission.'
                                                             'evaluation')

            f = open(summary_file, 'r')
            data = f.read()
            f.close()
            self.assertTrue('foo_dock ' in data)
            f = open(csv_file, 'r')
            data = f.read()
            f.close()
            self.assertTrue('foo_dock,2,3,4.000,9.000,6.500' in data)
        finally:
            shutil.rmtree(temp_dir)

    def test_main_success_cause_no_evaluations_to_check(self):
        temp_dir = tempfile.mkdtemp()

        try:
            theargs = ['post_evaluation.py', temp_dir]

            self.assertEqual(post_evaluation.main(theargs), 0)

        finally:
            shutil.rmtree(temp_dir)

    def test_main_with_eval_task(self):
        temp_dir = tempfile.mkdtemp()

        try:
            result_dir = os.path.join(temp_dir, 'result')
            os.makedirs(result_dir, mode=0o755)

            chall_dir = os.path.join(temp_dir, 'challdir')
            os.makedirs(chall_dir, mode=0o755)

            # write out the final.log file
            finalfile = os.path.join(chall_dir, post_evaluation.CHALL_FINAL_LOG)
            f = open(finalfile, 'w')
            f.write("""03/04/17 03:32:54: ============Start to work in this query protein: 5uub============
            03/04/17 03:32:55: Warning: Found multiple ligand for this protein:LMCSS-5uub_5un3-XYP.pdb
            03/04/17 03:32:55: Succsessfully generate this protein:LMCSS-5uub_5un3-XYP.pdb
            03/04/17 03:32:55: Ligand center for this case:LMCSS-5uub_5un3-XYP-lig.pdb is   38.448,   73.584,   58.680
            03/04/17 03:33:00: Succsessfully generate this protein:SMCSS-5uub_5fsj-OXY.pdb
            03/04/17 03:33:12: ============Start to work in this query protein: 5ur3============
            03/04/17 03:33:12: Succsessfully generate this protein:LMCSS-5ur3_3njq-NJQ.pdb
            03/04/17 03:33:12: Ligand center for this case:LMCSS-5ur3_3njq-NJQ-lig.pdb is  -27.326,  -19.578,  -43.347
            03/04/17 03:33:19: Succsessfully generate this protein:hiResHolo-5ur3_4p3h-25G.pdb
            03/04/17 03:33:25: Ligand center for this case:LMCSS-5uud_1qf2-TI3-lig.pdb is   38.308,   38.885,   -3.382
            03/04/17 03:33:40: Succsessfully generate this protein:hiTanimoto-5uud_1gxw-VAL.pdb""")
            f.flush()
            f.close()

            task_dir = os.path.join(temp_dir,
                                    'stage.7.foo_dock.extsubmission.evaluation')
            os.makedirs(task_dir, mode=0o755)

            # write out the pickle dir
            smcss = post_evaluation.SMCSS
            hi_tanimoto = post_evaluation.HI_TANIMOTO
            hi_resapo = post_evaluation.HI_RESAPO
            hi_resholo = post_evaluation.HI_RESHOLO
            apickle = os.path.join(task_dir, post_evaluation.RMSD_PICKLE)
            data = {'5ka3': {smcss: 1.0, hi_resholo: 2.0, hi_tanimoto: 'foo',
                             post_evaluation.LMCSS: 4.0},
                    '5uud': {hi_resholo: 5.0, smcss: 6.0, hi_resapo: 7.0,
                             hi_tanimoto: 8.0, post_evaluation.LMCSS: 9.0}}
            f = open(apickle, 'w')
            pickle.dump(data, f)
            f.flush()
            f.close()
            theargs = ['post_evaluation.py', result_dir,
                       '--evaluationdir', task_dir,
                       '--challengedir', chall_dir]

            self.assertEqual(post_evaluation.main(theargs), 0)

            summary_file = os.path.join(result_dir,
                                        post_evaluation.SUMMARY_TXT)
            self.assertTrue(os.path.isfile(summary_file))
            f = open(summary_file, 'r')
            data = f.read()
            f.close()

            self.assertTrue(post_evaluation.LMCSS + ' (2 dockable)' in data)
            self.assertTrue('foo_dock ' in data)
            self.assertTrue(' 2 (100%)    4.000       9.000' in data)

            csv_file = os.path.join(result_dir,
                                    post_evaluation.OVERALL_RMSD_PREFIX +
                                    post_evaluation.LMCSS +
                                    post_evaluation.CSV_SUFFIX)
            self.assertTrue(os.path.isfile(csv_file))
            f = open(csv_file, 'r')
            data = f.read()
            f.close()
            self.assertTrue('SubmissionID for LMCSS,# Docked,' in data)
            self.assertTrue('foo_dock,2,2,4.000,9.000,6.500' in data)

            # TODO check contents of other CSV files
        finally:
            shutil.rmtree(temp_dir)

    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()



__author__ = 'robswift'


class Base(object):
    def __init__(self):
        self.pdb_id = None                  # 4 letter PDB code
        self.sequences = []                 # a list of sequence objects, either Bio.SeqRecord or d3r.HitSequence
        self.sequence_count = 0             # an integer count of the number of unique sequences
        self.dock = []                      # A list of dockable ligand objects.
        self.do_not_call = []               # A list of non-dockable ligand objects.
        self.dock_count = 0                 # an integer count of the number of dockable ligands
        self.ligand_count = 0               # an integer count of the total no. ligands
        self.triage = None                  # a string, 'tosser' or None, whose value determines whether a ->
                                            # -> structure (target or test object) should be discarded ('tosser') or ->
                                            # retained for subsequent docking (None).
        self.reasons_to_triage = []         # a list that stores the reasons, or failed filtering criteria, leading ->
                                            # -> to a triage decision of 'tosser'. The value of each reason is ->
                                            # -> selected from the reasons dict by the function self.set_reason.
        self.triage_reasons = {
            1 : 'The percent identity is below the threshold',
            2 : 'The percent coverage is below the threshold',
            3 : 'The number of unique sequences is above the threshold',
            4 : 'The number of dockable ligands is above the threshold',
            5 : 'A BLAST search of the wwPDB only returned apo structures',
            6 : 'No dockable ligands bound',
            7 : 'The structure of the BLAST hit was determined by an unsuitable method',
            8 : 'The BLAST query returned no suitable hits',
            9 : 'Error reading the InChI of the dockable ligand corresponding to the input pre-release sequence',
            10 : 'The structure contains no dockable ligands',
            11 : 'Contains other than protein sequences'
        }

    def set_reason(self, selection):
        """
        Sets the value of 'reason', an attribute that explains why the 'triage' attribute was set to 'tosser'. In
        addition to setting a reason, this method also sets the value of 'triage' to 'tosser'. To set a reason, an
        integer argument between 1 and 9, whose value maps to a particular reason, is provided. Selection to reason
        mapping:
            1 -> The percent identity is below the threshold
            2 -> The percent coverage is below the threshold
            3 -> The number of unique sequences (chains) is above the threshold
            4 -> The number of dockable ligands is above the threshold
            5 -> There are no ligand-bound structures that meet the BLAST criteria available in the wwPDB
            6 -> No ligands are bound to the structure associated with the input pre-release sequence
            7 -> The structure of the BLAST hit was determined by an unsuitable method
            8 -> The BLAST query returned no suitable hits
            9 -> Error reading the InChI of the dockable ligand corresponding to the input pre-release sequence
            10 -> The structure contains no dockable ligands
            11 -> Contains other than protein sequences
        Note that if the selection is outside of the range 1 to 9, no reason is set. Once the value for reason has
        been initiated, it is not fixed, and can be reset.
        :param selection: (int)
        """
        if int(selection) > 11 or int(selection) < 1:
            pass
        else:
            self.triage = 'tosser'
            self.reasons_to_triage.append(self.triage_reasons[int(selection)])


__author__ = 'robswift'

import os
import sys
import re
import logging

from Bio.Alphabet import IUPAC
from Bio import SeqIO
from d3r.blast.base import Base
from d3r.blast.ligand import Ligand
from d3r.filter import filtering_sets as filtering_sets
from d3r.blast.hit_sequence import HitSequence

logger = logging.getLogger(__name__)

try:
    from Bio.PDB import *
except ImportError:
    logger.exception('Unable to import Bio.PDB Hit class may not work')


class RegDict(dict):
    def get_matching(self, event):
        return ((key, self[key]) for key in self if re.match(event, key))


class Hit(Base):
    """

    """

    pdb_dir = None
    pdb_dict = RegDict()

    @staticmethod
    def set_pdb_dir(pdb_path):
        if os.path.isdir(pdb_path):
            Hit.pdb_dir = pdb_path
        else:
            raise IOError("'{pdb_path}' is not a directory".format(pdb_path=pdb_path))

    @staticmethod
    def set_pdb_dict(fasta):
        """
        Each PDB ID is mapped to a list of sequences, one sequence for each of its chains. This information is stored
        in a class-wide default dictionary pdb_dict, with the following structure
        pdb_dict = { 'pdbid_chainid' : Bio.SeqRecord }
        :param fasta: path to the PDB sequences stored in FASTA format, i.e. "pdb_seqres.txt"
        """
        try:
            fasta_handle = open(fasta, 'r')
        except IOError:
            logger.exception('Caught exeption opening fasta file')
            sys.exit(1)
        logger.debug('Parsing ' + fasta)
        for record in SeqIO.parse(fasta_handle, "fasta"):
            # only add protein sequences
            if 'mol:protein' in record.description:
                record.seq.alphabet = IUPAC.protein
                Hit.pdb_dict[record.id] = record
            elif 'mol:na' in record.description:
                Hit.pdb_dict[record.id] = record
                # default SingleLetterAlphabet() assigned

        fasta_handle.close()

    def __init__(self):
        super(Hit, self).__init__()
        self.retain = None              # True if the test object is retained for docking, False if not.
        self.reasons_to_retain = []     # A list that stores the reasons the test object was retained for docking.
        self.pdb = None                 # Bio.PDB structure object
        self.chain_count = 0            # an integer count of the number of PDB chains, which may correspond to -->
                                        # --> identical sequences.
        self.resolution = None          # structural resolution (angstroms)
        self.exp_method = None          # Method used to determine the structure, e.g. x-ray diffraction'
        self.largest_mcss = None        # The the largest MCSS contained in the self.dock.mcsss list
        self.largest_mcss_chain = []    # The chain ID of the largest MCSS
        self.largest_index = []         # The index of the largest MCSS in the self.dock list
        self.smallest_mcss = None       # The smallest MCSS contained in the self.dock.mcsss list
        self.smallest_mcss_chain = []   # The chain ID of the smallest MCSS
        self.smallest_index = []        # The index of the smallest MCSS in the self.dock list
        self.highest_tanimoto = None    # The tanimoto similarity contained in the self.dock.mcsss list
        self.highest_tanimoto_chain = []# The chain ID of the highest tanimoto
        self.highest_tanimoto_index = []# The index of the highest tanimoto in the self.dock list
        self.sequence_membership = {}   # {'pdb_id'_'chain_id' : index} where index is the index of the -->
                                        # --> corresponding HitSequence object in the sequences list.

        #add category 5 for largest tanimoto score 09/01 sliu. 
        self.retain_reasons = {
            1: 'The BLAST hit is bound to the ligand with the largest maximum common substructure',
            2: 'The BLAST hit is bound to the ligand with the smallest maximum common substructure',
            3: 'The BLAST hit is the highest resolution holo structure',
            4: 'The BLAST hit is the highest resolution apo structure',
            5: 'The BLAST hit is bound to the ligand with the highest tanimoto score in structural similarity',
        }

    def set_retain_reason(self, selection):
        """
        Sets the value of 'retain_reason', an attribute that explains why the test object is being retained for docking.
        In addition to setting a reason, this method also sets the value of 'retain' to True. To set a reason, an
        integer argument between 1 and 4, whose value maps to a particular reason, is provided. Selection to reason
        mapping:
            1 -> The BLAST hit is bound to the ligand with the largest maximum common substructure
            2 -> The BLAST hit is bound to the ligand with the smallest maximum common substructure
            3 -> The BLAST hit is the highest resolution structure
            4 -> The BLAST hit is the highest resolution apo structure
            5 -> The BLAST hit is bound to the ligand with the highest tanimoto score in structural similarity
        Note that if the selection is outside of the range 1 to 5, no reason is set. Once the value for reason has
        been initiated, it is not fixed, and can be reset.
        :param selection: (int)
        """
        #add category 5 for largest tanimoto score 09/01 sliu. 
        if int(selection) > 5 or int(selection) < 1:
            pass
        else:
            self.retain = True
            reason = self.retain_reasons[int(selection)]
            if reason not in self.reasons_to_retain:
                self.reasons_to_retain.append(reason)

    def set_maxmin_mcss(self):
        """
        Finds the largest and smallest MCSS objects contained in the list of dockable ligands. If this list of ligands
        doesn't exist, or if none of the ligands contain an MCSS, the method does not execute.
        """
        if not self.dock:
            pass
        elif not self.dock[0].mcsss:
            pass
        else:
            self.largest_mcss = self.dock[0].mcsss[0]
            self.smallest_mcss = self.dock[0].mcsss[0]
            #add highest tanimoto score 09/01 sliu
            self.highest_tanimoto = self.dock[0].mcsss[0]
            self.largest_mcss_chain = []
            self.smallest_mcss_chain = []
            self.highest_tanimoto_chain = []
            #store the index of the largest mcss in the self.dock object 
            self.largest_index = []
            #store the index of the smallest mcss in the self.dock object 
            self.smallest_index = [] 
            self.highest_tanimoto_index = [] 
            for (ligand_index, ligand) in enumerate (self.dock):
                    #modified by sliu 08/04 to add chain info
                    for mcss in ligand.mcsss:
                        if mcss.size > self.largest_mcss.size:
                            self.largest_mcss = mcss
                            #if this mcss is bigger then clean up the chain list and pick this one  
                            self.largest_mcss_chain = []
                            self.largest_mcss_chain.append(ligand.chain)
                            self.largest_index = []
                            self.largest_index.append(ligand_index)
                        if mcss.size == self.largest_mcss.size:
                            #if this mcss is the same as the original one, just append
                            self.largest_mcss_chain.append(ligand.chain)
                            self.largest_index.append(ligand_index)
                        if mcss.size < self.largest_mcss.size:
                            self.smallest_mcss = mcss
                            self.smallest_mcss_chain = []
                            self.smallest_mcss_chain.append(ligand.chain)      
                            self.smallest_index = []                                
                            self.smallest_index.append(ligand_index)
                        if mcss.size == self.smallest_mcss.size:
                            self.smallest_mcss_chain.append(ligand.chain)
                            self.smallest_index.append(ligand_index)
                        if mcss.tanimoto > self.highest_tanimoto.tanimoto:
                            self.highest_tanimoto = mcss
                            self.highest_tanimoto_chain = []
                            self.highest_tanimoto_chain.append(ligand.chain)
                            self.highest_tanimoto_index = []
                            self.highest_tanimoto_index.append(ligand_index)
                        if mcss.tanimoto == self.highest_tanimoto.tanimoto:
                            self.highest_tanimoto_chain.append(ligand.chain)
                            self.highest_tanimoto_index.append(ligand_index)
                         
            #sort the chain list so that later we could anyway pick the first one.
            (self.largest_mcss_chain, self.largest_index) = (list(x) for x in zip(*sorted(zip(self.largest_mcss_chain, self.largest_index))))
            #print "CCCCCCCCCCCCCCC", (self.largest_mcss_chain, self.largest_index)  
            (self.smallest_mcss_chain, self.smallest_index) = (list(x) for x in zip(*sorted(zip(self.smallest_mcss_chain, self.smallest_index))))
            #here we need to clean up the self.dock to only reserve one ligand
            (self.highest_tanimoto_chain, self.highest_tanimoto_index) = (list(x) for x in zip(*sorted(zip(self.highest_tanimoto_chain, self.highest_tanimoto_index))))

    def read_pdb(self):
        """
        Reads the PDB file corresponding to the Test objects wwPDB ID, self.pdb_id, and stores the results as a
        Bio.PDB structure object. If reading the file and assigning the Bio.PDB structure object is successful, True
        is returned. Otherwise, False is returned. Prior to calling the method, the pdb_id attribute must be set. E.g.
        typical usage follows
            test = d3r.blast.Test()
            test.pdb_id = '1xdn'
            test.read_pdb()
        :return: Boolean
        """
        logger.debug('In read_pdb()')
        if not self.pdb_id:
            logger.error("Test.pdb_id must be set prior to calling the read_pdb method")
            return False
        pdb_file = os.path.join(Hit.pdb_dir, self.pdb_id[1:3], 'pdb' + self.pdb_id + '.ent')
        try:
            handle = open(pdb_file, 'r')
            handle.close()
        except IOError as e:
            logger.exception('Problems when assigning the ligrand for wwPDB ID: ' + self.pdb_id)
            return False
        try:
            parser = PDBParser()
            self.pdb = parser.get_structure(format(self.pdb_id), pdb_file)
            if not self.pdb:
                logger.error('Unable to get structure for pdb')
                return False
            else:
                return True
        except:
            logger.exception('Caught exception')
            return False

    def set_coverage(self, coverage):
        self.coverage = float(coverage)

    def set_identity(self, identity):
        self.identity = float(identity)

    def set_sequence(self, pdb_id, chain_id, record, alignment):
        """
        Looks for pdb sequence in the contents of the 'pdb_seqres.txt' file, stored in pdb_dict
        :param pdb_id: 4-letter pdb id
        """

        id = '{pdb_id}_{chain_id}'.format(pdb_id = pdb_id.lower(), chain_id = chain_id.upper())
        #sliu2 on 01/09/2017 fix the bug where the sequence chain is lower case in sequence file
        if id not in Hit.pdb_dict:
            #if the upper chain id is not found then try to lower chain id
            id = '{pdb_id}_{chain_id}'.format(pdb_id = pdb_id.lower(), chain_id = chain_id.lower())
            if id not in Hit.pdb_dict:
                # if the lower chain id is still not avaiable, in this case, cannot set the sequence
                return False
        logger.debug('Looking for ' + id + ' in pdb_seqres.txt')
        if id in self.sequence_membership.keys():
            self.sequences[self.sequence_membership[id]].set_query_alignment(record, alignment)
        else:
            seq_record = Hit.pdb_dict[id]
            if self.unique_sequence(seq_record.seq):
                self.sequence_count += 1
                hs = HitSequence(seq_record)
                hs.hit_sequence_id = self.sequence_count
            else:
                seq_id = [hs.hit_sequence_id for hs in self.sequences if hs.seq_record.seq == seq_record.seq][0]
                hs = HitSequence(seq_record)
                hs.hit_sequence_id = seq_id
            hs.blast_hit = True
            hs.hit_pdb_id = pdb_id
            hs.hit_chain_id = chain_id
            hs.set_query_alignment(record, alignment)
            self.sequences.append(hs)
            self.sequence_membership[id] = len(self.sequences) - 1
            self.chain_count += 1
            return True

    def fill_sequence(self):
        """
        Fills in chains present in the pdb that weren't picked up by blast. For example, nucleic acid
        components co-crystallized with the protein would be picked up by the fill_sequence method. This should only
        be called after the set_hits method is called
        """
        logger.debug('In fill_sequence()')
        for id, seq_record in Hit.pdb_dict.get_matching(self.pdb_id):
            if id in self.sequence_membership.keys():
                # the chain exists in the sequence list, so no need to do anything
                continue
            else:
                # the chain hasn't been assigned. There are two possibilities here.
                # i) the chain corresponds to a sequence that has already been assigned.
                # ii) the chain corresponds to a new sequence
                # case i) is dealt with first
                if self.unique_sequence(seq_record.seq):
                    self.sequence_count += 1
                    hs = HitSequence(seq_record)
                    hs.hit_sequence_id = self.sequence_count
                else:
                    seq_id = [hs.hit_sequence_id for hs in self.sequences if hs.seq_record.seq == seq_record.seq][0]
                    hs = HitSequence(seq_record)
                    hs.hit_sequence_id = seq_id
                hs.blast_hit = False
                pdb_id, chain_id = id.split('_')
                hs.hit_pdb_id = pdb_id
                hs.hit_chain_id = chain_id
                self.sequences.append(hs)
                self.sequence_membership[id] = len(self.sequences) - 1
                self.chain_count += 1

    def unique_sequence(self, seq):
        """
        Returns true if the input sequence cannot be found in the list of Hit object's sequences, otherwise returns
        False.
        :param seq: a Bio.Seq object
        :return: Boolean
        """
        if len([hs for hs in self.sequences if hs.seq_record.seq == seq]) == 0:
            return True
        else:
            return False

    def set_ligands(self, chain_id):
        """
        Creates a d3r.blast.Ligand object for each of the bound ligands found in the PDB, and appends
        this object to the appropriate list, either dock or do_not_call. If the ligand object is added
        to the dock list, the dock_count and the ligand_count is incremented. If the ligand object is added to the
        do_not_call list, then the ligand_count is incremented. No effort is made to check whether or not the
        input ligand is unique, i.e. that it hasn't been added before. Prior to calling the method, the pdb_id
        attribute must be set, and the read_pdb method must be called. For example:
            test = d3r.blast.Test()
            test.pdb_id = '1xdn'
            test.read_pdb()
            test.set_ligands()
        """
        logger.debug('In set_ligands()')
        model_list = Selection.unfold_entities(self.pdb, 'M')
        chain_list = Selection.unfold_entities(model_list[0], 'C')
        chain_format = "<Chain id=%s>"%chain_id
        chain_index = 0 
        for (index, chain_object) in enumerate(chain_list):
            if str(chain_object) == chain_format:
                chain_index = index
        #print "Chain index", chain_index
        res_list = Selection.unfold_entities(chain_list[chain_index], 'R')
        hetero_list = [res for res in res_list if 'H_' in res.id[0] and res.id[0] != 'H_MSE']
        #modified by sliu 08/04, add chain info for ligand
        for hetero in hetero_list:
            assigned = []
            for l in self.dock:
                ligand_info = (l.resname, l.chain)
                assigned.append(ligand_info)
            for l in self.do_not_call:
                ligand_info = (l.resname, l.chain)
                assigned.append(ligand_info)

            resname = hetero.resname.strip()
            if (resname, chain_id) in assigned:
                #the resname, chain id info is already in stored
                continue
            ligand = Ligand()
            ligand.resname = resname
            ligand.chain = chain_id
            if resname in filtering_sets.do_not_call:
                ligand.label = 'do_not_call'
                self.do_not_call.append(ligand)
                self.ligand_count += 1
            else:
                ligand.label = 'dock'
                ligand.set_rd_mol_from_resname(resname)
                #modified by sliu 08/08
                ligand.set_size()
                ligand.set_heavy_size()
                self.dock.append(ligand)
                self.dock_count += 1
                self.ligand_count += 1

    def set_resolution(self):
        """
        Set the resolution of the PDB of the test object, self.resolution. Before this method is called, the pdb_id
        attribute must be set, and the read_pdb method must be called. For example:
            test = d3r.blast.Test()
            test.pdb_id = '1xdn'
            test.read_pdb()
            test.set_resolution()
        If the resolution cannot be set, False is returned, otherwise True is returned.
        :return: Boolean
        """
        logger.debug('In set_resolution()')
        if not self.pdb_id or not self.pdb:
            logger.error("Before the set_resolution method is called, the pdb_id attribute must be set, "
                         "and the read_pdb "
                         "method must be called.")
            return False
        # TO DO: add error checking for NMR structures, and other experimental methods from which resolution is
        # unavailable
        try:
            self.resolution = self.pdb.header['resolution']
            if self.resolution:
                return True
            else:
                return False
        except:
            logger.exception('Caught exception')
            return False

    def set_expt_method(self):
        """
        Set the experimental structure determination method of the PDB of the test object, self.exp_method. Before this
        method is called, the pdb_id attribute must be set, and the read_pdb method must be called. For example:
            test = d3r.blast.Test()
            test.pdb_id = '1xdn'
            test.read_pdb()
            test.set_expt_method()
        If the resolution cannot be set, False is returned, otherwise True is returned.
        :return: Boolean
        """
        logger.debug('In set_expt_method()')
        if not self.pdb_id or not self.pdb:
            logger.error("Before the set_expt_method method is called, the pdb_id attribute must be set, and the read_pdb "
                         "method must be called.")
            return False
        try:
            self.exp_method = self.pdb.header['structure_method']
            if self.resolution:
                return True
            else:
                return False
        except:
            return False

    def get_coverage(self):
        return self.coverage

    def get_identity(self):
        return self.identity

    def get_sequences(self):
        return self.sequences

    def get_resolution(self):
        return self.resolution




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_custom_dock
----------------------------------

Tests for `custom_dock` module.
"""

import unittest
import tempfile
import shutil
import os
import time

from d3r.celppade.custom_dock import Dock


class TestDock(unittest.TestCase):

    def setUp(self):
        pass

    def test_dock(self):
        """Test where we run dock() with fake arguments
           to verify we always get False
        """
        d = Dock()
        self.assertEqual(d.dock(1, 2, 3, 4), False)

    def test_ligand_technical_prep(self):
        """Test where we run ligand_technical_prep() with fake argument
           to verify we always get back the argument as an array
        """
        d = Dock()
        self.assertEqual(d.ligand_technical_prep('hi'), ['hi'])

    def test_receptor_technical_prep(self):
        """Test where we run receptor_technical_prep() with fake arguments
           to verify we always get back the 1st argument as an array
        """
        d = Dock()
        self.assertEqual(d.receptor_technical_prep('hi', 'foo'), ['hi'])

    def test_get_pocket_center(self):
        temp_dir = tempfile.mkdtemp()
        try:
            d = Dock()
            # test where file does not exist
            self.assertEqual(d.get_pocket_center(temp_dir), False)
        finally:
            shutil.rmtree(temp_dir)    


        temp_dir = tempfile.mkdtemp()
        try:
            d = Dock()
            # test with valid center.txt file
            pcfile = os.path.join(temp_dir, 'center.txt')
            f = open(pcfile, 'w')
            f.write('1,2')
            f.flush()
            f.close()
            self.assertEqual(d.get_pocket_center(temp_dir), False)
        finally:
            shutil.rmtree(temp_dir)    


        temp_dir = tempfile.mkdtemp()
        try:
            d = Dock()
            # test where file does not exist
            self.assertEqual(d.get_pocket_center(temp_dir), False)
            pcfile = os.path.join(temp_dir, 'center.txt')
            f = open(pcfile, 'w')
            f.write('1,2,3')
            f.flush()
            f.close()
            self.assertEqual(d.get_pocket_center(temp_dir), [1.0, 2.0, 3.0])
        finally:
            shutil.rmtree(temp_dir)    


        temp_dir = tempfile.mkdtemp()
        try:
            d = Dock()
            # test where file does not exist
            self.assertEqual(d.get_pocket_center(temp_dir), False)
            pcfile = os.path.join(temp_dir, 'center.txt')
            f = open(pcfile, 'w')
            f.write('a,b,c')
            f.flush()
            f.close()
            self.assertEqual(d.get_pocket_center(temp_dir), False)
        finally:
            shutil.rmtree(temp_dir)    

    def test_get_sci_prepped_lig(self):
        # test where file does not exist
        temp_dir = tempfile.mkdtemp()
        try:
            d = Dock()
            self.assertEqual(d.get_sci_prepped_lig(temp_dir,d.SCI_PREPPED_LIG_SUFFIX), False)
        finally:
            shutil.rmtree(temp_dir)    

        # Test where correct file exists
        temp_dir = tempfile.mkdtemp()
        try:
            d = Dock()
            ligfile = os.path.join(temp_dir, 'lig_TST'+d.SCI_PREPPED_LIG_SUFFIX)
            f = open(ligfile, 'w')
            f.write('There is no test to ensure the contents are valid. This string should pass though.')
            f.flush()
            f.close()
            self.assertEqual(d.get_sci_prepped_lig(temp_dir,d.SCI_PREPPED_LIG_SUFFIX), ligfile)
        finally:
            shutil.rmtree(temp_dir)    


        # Test where file has wrong suffix
        temp_dir = tempfile.mkdtemp()
        try:
            d = Dock()
            ligfile = os.path.join(temp_dir, 'lig_TST_prepared.xxx')
            f = open(ligfile, 'w')
            f.write('There is no test to ensure the contents are valid. This string should pass though.')
            f.flush()
            f.close()
            self.assertEqual(d.get_sci_prepped_lig(temp_dir,d.SCI_PREPPED_LIG_SUFFIX), False)
        finally:
            shutil.rmtree(temp_dir)    


        # Test where multiple correct files exist
        temp_dir = tempfile.mkdtemp()
        try:
            d = Dock()
            ligfile = os.path.join(temp_dir, 'lig_TST'+d.SCI_PREPPED_LIG_SUFFIX)
            f = open(ligfile, 'w')
            f.write('There is no test to ensure the contents are valid. This string should pass though.')
            f.flush()
            f.close()

            ligfile = os.path.join(temp_dir, 'lig_TS2'+d.SCI_PREPPED_LIG_SUFFIX)
            f = open(ligfile, 'w')
            f.write('There is no test to ensure the contents are valid. This string should pass though.')
            f.flush()
            f.close()
            self.assertEqual(d.get_sci_prepped_lig(temp_dir,d.SCI_PREPPED_LIG_SUFFIX), False)
        finally:
            shutil.rmtree(temp_dir)    


    def test_parse_lig_filename(self):
        d = Dock()

        # Blank name
        self.assertEqual(d.parse_lig_filename(''), False)

        # Good name
        ligfile = os.path.join('some_directory', 'lig_TST'+d.SCI_PREPPED_LIG_SUFFIX)
        self.assertEqual(d.parse_lig_filename(ligfile), 'TST')

        # Good name, but in list
        self.assertEqual(d.parse_lig_filename([ligfile]), False)

        # Wrong suffix
        ligfile = os.path.join('some_directory', 'lig_TST_prepared.xxx')
        self.assertEqual(d.parse_lig_filename(ligfile), False)

        # Change expected suffix to "_prepared.xxx"
        d.SCI_PREPPED_LIG_SUFFIX = '_prepared.xxx'
        self.assertEqual(d.parse_lig_filename(ligfile), 'TST')


    def test_parse_cand_name(self):
        d = Dock()
        
        # Blank name
        self.assertEqual(d.parse_cand_name(''), False)
        
        # Good name
        candfile = os.path.join('some_directory', 'hiResHolo-1fcz_1fcy'+d.SCI_PREPPED_PROT_SUFFIX)
        self.assertEqual(d.parse_cand_name(candfile), ('hiResHolo','1fcz','1fcy'))
        
        # Good name, but in list
        self.assertEqual(d.parse_cand_name([candfile]), False)
        
        # Wrong suffix
        candfile = os.path.join('some_directory', 'hiResHolo-1fcz_1fcy_prepared.xxx')
        self.assertEqual(d.parse_cand_name(candfile), False)
        
        # Change to be right suffix
        d.SCI_PREPPED_PROT_SUFFIX = '_prepared.xxx'
        self.assertEqual(d.parse_cand_name(candfile), ('hiResHolo','1fcz','1fcy'))

        
    def test_run_dock(self):
        orig_dir = os.getcwd()
        temp_dir = tempfile.mkdtemp()
        temp_dir_abs = os.path.abspath(temp_dir)
        prot_sci_prep_dir = os.path.join(temp_dir,'prot_sci_prep_dir')
        lig_sci_prep_dir = os.path.join(temp_dir,'lig_sci_prep_dir')
        dock_dir = os.path.join(temp_dir,'dock_dir')
        try:        
            os.mkdir(prot_sci_prep_dir)
            sample_prot_dir = os.path.join(prot_sci_prep_dir,'1fcz')
            os.mkdir(sample_prot_dir)
            sample_prot_file = os.path.join(sample_prot_dir, 'hiResHolo-1fcz_1fcy_prepared.pdb')
            with open(sample_prot_file, 'wb') as of:
                of.write('test')
            sample_center_file = os.path.join(sample_prot_dir, 'center.txt')
            with open(sample_center_file, 'wb') as of:
                of.write('1,2,3')
            sample_cand_targinfo_file = os.path.join(sample_prot_dir, '1fcz.txt')
            with open(sample_cand_targinfo_file, 'wb') as of:
                of.write('query, 1fcz')
        
            os.mkdir(lig_sci_prep_dir)
            sample_lig_dir = os.path.join(lig_sci_prep_dir,'1fcz')
            os.mkdir(sample_lig_dir)
            sample_lig_file = os.path.join(sample_lig_dir, 'lig_156_prepared.sdf')
            with open(sample_lig_file, 'wb') as of:
                of.write('test')
            sample_lig_targinfo_file = os.path.join(sample_lig_dir, '1fcz.txt')
            with open(sample_lig_targinfo_file, 'wb') as of:
                of.write('query, 1fcz')            
            os.mkdir(dock_dir)
            d = Dock()
            d.run_dock(prot_sci_prep_dir, lig_sci_prep_dir, dock_dir)
            
            dock_targ_dir = os.path.join(dock_dir,'1fcz')
            self.assertEqual(os.path.isdir(dock_targ_dir), True)
            dock_cand_prep_dir = os.path.join(dock_targ_dir,'hiResHolo_1fcy_tech_prep')
            self.assertEqual(os.path.isdir(dock_cand_prep_dir), True)
            dock_lig_prep_dir = os.path.join(dock_targ_dir,'lig_156_tech_prep')
            self.assertEqual(os.path.isdir(dock_lig_prep_dir), True)
            dock_cand_dock_dir = os.path.join(dock_targ_dir,'hiResHolo_1fcy_docking')
            self.assertEqual(os.path.isdir(dock_cand_dock_dir), True)
            dock_lig_file = os.path.join(dock_cand_dock_dir,'lig_156_prepared.sdf')
            self.assertEqual(os.path.exists(dock_lig_file), True)
            dock_cand_file = os.path.join(dock_cand_dock_dir,'hiResHolo-1fcz_1fcy_prepared.pdb')
            self.assertEqual(os.path.exists(dock_cand_file), True)
        finally:
            os.chdir(orig_dir)
            shutil.rmtree(temp_dir_abs)

   
           
    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()



__author__ = 'robswift'

from d3r.blast.hit import Hit

def filter_blast_result(alignment, record):
    """
    The binding pocket of the target structure should be covered by identical residues in the test sequence. We
    don't know where the target-sequence binding pocket is located. To increase the chances that the test sequence
    and target sequence have identical binding pockets, this function checks the following criteria:
    (i) X % of the target sequence must have a test-sequence alignment partner. Where X is given by the class
    variable, length threshold.
    (ii) Y % of the aligned sequence must be identical to the target sequence. Where Y is given by the class
    variable, identity_threshold.
    :param alignment: Bio.blast.Record.Alignment object
    :param record: Bio.blast.Record object
    :return: True if criteria are satisfied otherwise return False
    """
    length_threshold = 0.9
    identity_threshold = 0.95
    minimum_coverage = round(length_threshold * record.query_length)        # the smallest no. of target sequence
                                                                            # AA that align with the test sequence.
    minimum_identity = round(minimum_coverage * identity_threshold)         # the smallest no. of identical AA in
                                                                            # the alignment.
    # sum over the HSP (High Scoring Pairs of aligned test sequence residues)
    aligned_test_seq_length = 0
    identical_test_seq_length = 0
    for hsp in alignment.hsps:
        aligned_test_seq_length += hsp.align_length
        identical_test_seq_length += hsp.identities

    # check criteria
    if (minimum_coverage <= aligned_test_seq_length) & (minimum_identity <= identical_test_seq_length):
        return True
    else:
        return False


__author__ = 'churas'

"""
test_vinadocking
--------------------------------

Tests for `vinadocking` module.
"""

import unittest
import os
import stat
import tempfile
import shutil

from d3r import chimera_proteinligprep


class fakeparams(object):
    """Holds fake parameters
    """
    pass


class TestChimera_ProteinLigPrep(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_ligand_prepare_ligfile_already_prepared(self):
        temp_dir = tempfile.mkdtemp()
        try:
            ligfile = os.path.join(temp_dir, 'ligfile')
            open(ligfile, 'a').close()
            val = chimera_proteinligprep.ligand_prepare(None, ligfile, None)
            self.assertTrue(val)
        finally:
            shutil.rmtree(temp_dir)

    def test_ligand_prepare_with_fake_rdkit_python(self):
        temp_dir = tempfile.mkdtemp()
        curdir = os.getcwd()
        try:
            os.chdir(temp_dir)
            ligfile = os.path.join(temp_dir, 'ligfile')
            lig_smile = os.path.join(temp_dir, '1fcz.smi')

            temp_bin_dir = os.path.join(temp_dir, 'bin')
            os.makedirs(temp_bin_dir)

            fakepython = os.path.join(temp_bin_dir, 'python')

            f = open(fakepython, 'w')
            f.write('#!/usr/bin/env python\n\n')
            f.write('import sys\n')
            f.write('f = open(sys.argv[2], "w")\n')
            f.write('f.write(sys.argv[1] + ":")\n')
            f.write('f.write(sys.argv[2] + ":")\n')
            f.write('f.write(sys.argv[3])\n')
            f.write('f.close()\n')
            f.write('print sys.argv[1]\n')
            f.write('sys.exit(0)\n')
            f.flush()
            f.close()
            os.chmod(fakepython, stat.S_IRWXU)

            val = chimera_proteinligprep.ligand_prepare(lig_smile,
                                                        ligfile, temp_bin_dir)
            self.assertFalse(val)
            f = open(lig_smile, 'r')
            line = f.readline()
            f.close()
            unprep = lig_smile.replace('.smi','_unprep_step1.sdf')
            self.assertEqual(line, 'rdkit_smiles_to_3d_sdf.py:' +
                             lig_smile + ':' + unprep)
            out = os.path.join(temp_dir, 'rdkit_smiles_to_3d_sdf_out')
            self.assertTrue(os.path.isfile(out))
            f = open(out, 'r')
            line = f.readline()
            f.close()
            self.assertEqual(line, 'rdkit_smiles_to_3d_sdf.py\n')
        finally:
            os.chdir(curdir)
            shutil.rmtree(temp_dir)


if __name__ == '__main__':
    unittest.main()
